@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """On-disk mutex protecting a resource
 
@@ -21,7 +21,7 @@ particular name within the control directory.  We use this rather than OS
 internal locks (such as flock etc) because they can be seen across all
 transports, including http.
 
-Objects can be read if there is only physical read access; therefore 
+Objects can be read if there is only physical read access; therefore
 readers can never be required to create a lock, though they will
 check whether a writer is using the lock.  Writers can't detect
 whether anyone else is reading from the resource as they write.
@@ -56,7 +56,7 @@ rename-and-overwrite, making it hard to tell who won.)
 
 The desired characteristics are:
 
-* Locks are not reentrant.  (That is, a client that tries to take a 
+* Locks are not reentrant.  (That is, a client that tries to take a
   lock it already holds may deadlock or fail.)
 * Stale locks can be guessed at by a heuristic
 * Lost locks can be broken by any client
@@ -78,7 +78,7 @@ object per actual lock on disk.  This module does nothing to prevent aliasing
 and deadlocks will likely occur if the locks are aliased.
 
 In the future we may add a "freshen" method which can be called
-by a lock holder to check that their lock has not been broken, and to 
+by a lock holder to check that their lock has not been broken, and to
 update the timestamp within it.
 
 Example usage:
@@ -105,33 +105,37 @@ Example usage:
 
 import os
 import time
-from cStringIO import StringIO
 
 from bzrlib import (
     debug,
     errors,
+    lock,
+    osutils,
     )
 import bzrlib.config
+from bzrlib.decorators import only_raises
 from bzrlib.errors import (
         DirectoryNotEmpty,
         FileExists,
         LockBreakMismatch,
         LockBroken,
         LockContention,
+        LockCorrupt,
         LockFailed,
         LockNotHeld,
         NoSuchFile,
         PathError,
         ResourceBusy,
         TransportError,
-        UnlockableTransport,
         )
 from bzrlib.trace import mutter, note
-from bzrlib.transport import Transport
-from bzrlib.osutils import rand_chars, format_delta
-from bzrlib.rio import read_stanza, Stanza
+from bzrlib.osutils import format_delta, rand_chars, get_host_name
 import bzrlib.ui
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import rio
+""")
 
 # XXX: At the moment there is no consideration of thread safety on LockDir
 # objects.  This should perhaps be updated - e.g. if two threads try to take a
@@ -148,12 +152,13 @@ import bzrlib.ui
 # files/dirs created.
 
 
-_DEFAULT_TIMEOUT_SECONDS = 300
+_DEFAULT_TIMEOUT_SECONDS = 30
 _DEFAULT_POLL_SECONDS = 1.0
 
 
-class LockDir(object):
-    """Write-lock guarding access to data."""
+class LockDir(lock.Lock):
+    """Write-lock guarding access to data.
+    """
 
     __INFO_NAME = '/info'
 
@@ -164,11 +169,9 @@ class LockDir(object):
 
         :param transport: Transport which will contain the lock
 
-        :param path: Path to the lock within the base directory of the 
+        :param path: Path to the lock within the base directory of the
             transport.
         """
-        assert isinstance(transport, Transport), \
-            ("not a transport: %r" % transport)
         self.transport = transport
         self.path = path
         self._lock_held = False
@@ -191,7 +194,7 @@ class LockDir(object):
     def create(self, mode=None):
         """Create the on-disk lock.
 
-        This is typically only called when the object/directory containing the 
+        This is typically only called when the object/directory containing the
         directory is first created.  The lock is not held when it's created.
         """
         self._trace("create lock directory")
@@ -203,7 +206,7 @@ class LockDir(object):
 
     def _attempt_lock(self):
         """Make the pending directory and attempt to rename into place.
-        
+
         If the rename succeeds, we read back the info file to check that we
         really got the lock.
 
@@ -240,9 +243,17 @@ class LockDir(object):
         # incorrect.  It's possible some other servers or filesystems will
         # have a similar bug allowing someone to think they got the lock
         # when it's already held.
+        #
+        # See <https://bugs.launchpad.net/bzr/+bug/498378> for one case.
+        #
+        # Strictly the check is unnecessary and a waste of time for most
+        # people, but probably worth trapping if something is wrong.
         info = self.peek()
         self._trace("after locking, info=%r", info)
-        if info['nonce'] != self.nonce:
+        if info is None:
+            raise LockFailed(self, "lock was renamed into place, but "
+                "now is missing!")
+        if info.get('nonce') != self.nonce:
             self._trace("rename succeeded, "
                 "but lock is still held by someone else")
             raise LockContention(self)
@@ -254,7 +265,7 @@ class LockDir(object):
     def _remove_pending_dir(self, tmpname):
         """Remove the pending directory
 
-        This is called if we failed to rename into place, so that the pending 
+        This is called if we failed to rename into place, so that the pending
         dirs don't clutter up the lockdir.
         """
         self._trace("remove %s", tmpname)
@@ -286,6 +297,7 @@ class LockDir(object):
                                             info_bytes)
         return tmpname
 
+    @only_raises(LockNotHeld, LockBroken)
     def unlock(self):
         """Release a held lock
         """
@@ -293,11 +305,12 @@ class LockDir(object):
             self._fake_read_lock = False
             return
         if not self._lock_held:
-            raise LockNotHeld(self)
+            return lock.cant_unlock_not_held(self)
         if self._locked_via_token:
             self._locked_via_token = False
             self._lock_held = False
         else:
+            old_nonce = self.nonce
             # rename before deleting, because we can't atomically remove the
             # whole tree
             start_time = time.time()
@@ -323,6 +336,10 @@ class LockDir(object):
                 self.transport.delete_tree(tmpname)
             self._trace("... unlock succeeded after %dms",
                     (time.time() - start_time) * 1000)
+            result = lock.LockResult(self.transport.abspath(self.path),
+                                     old_nonce)
+            for hook in self.hooks['lock_released']:
+                hook(result)
 
     def break_lock(self):
         """Break a lock not held by this instance of LockDir.
@@ -332,12 +349,18 @@ class LockDir(object):
         it possibly being still active.
         """
         self._check_not_locked()
-        holder_info = self.peek()
+        try:
+            holder_info = self.peek()
+        except LockCorrupt, e:
+            # The lock info is corrupt.
+            if bzrlib.ui.ui_factory.get_boolean("Break (corrupt %r)" % (self,)):
+                self.force_break_corrupt(e.file_data)
+            return
         if holder_info is not None:
             lock_info = '\n'.join(self._format_lock_info(holder_info))
             if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
                 self.force_break(holder_info)
-        
+
     def force_break(self, dead_holder_info):
         """Release a lock held by another process.
 
@@ -351,7 +374,7 @@ class LockDir(object):
         LockBreakMismatch is raised.
 
         After the lock is broken it will not be held by any process.
-        It is possible that another process may sneak in and take the 
+        It is possible that another process may sneak in and take the
         lock before the breaking process acquires it.
         """
         if not isinstance(dead_holder_info, dict):
@@ -366,7 +389,7 @@ class LockDir(object):
         tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
         self.transport.rename(self._held_dir, tmpname)
         # check that we actually broke the right lock, not someone else;
-        # there's a small race window between checking it and doing the 
+        # there's a small race window between checking it and doing the
         # rename.
         broken_info_path = tmpname + self.__INFO_NAME
         broken_info = self._read_info_file(broken_info_path)
@@ -374,6 +397,39 @@ class LockDir(object):
             raise LockBreakMismatch(self, broken_info, dead_holder_info)
         self.transport.delete(broken_info_path)
         self.transport.rmdir(tmpname)
+        result = lock.LockResult(self.transport.abspath(self.path),
+                                 current_info.get('nonce'))
+        for hook in self.hooks['lock_broken']:
+            hook(result)
+
+    def force_break_corrupt(self, corrupt_info_lines):
+        """Release a lock that has been corrupted.
+        
+        This is very similar to force_break, it except it doesn't assume that
+        self.peek() can work.
+        
+        :param corrupt_info_lines: the lines of the corrupted info file, used
+            to check that the lock hasn't changed between reading the (corrupt)
+            info file and calling force_break_corrupt.
+        """
+        # XXX: this copes with unparseable info files, but what about missing
+        # info files?  Or missing lock dirs?
+        self._check_not_locked()
+        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
+        # check that we actually broke the right lock, not someone else;
+        # there's a small race window between checking it and doing the
+        # rename.
+        broken_info_path = tmpname + self.__INFO_NAME
+        f = self.transport.get(broken_info_path)
+        broken_lines = f.readlines()
+        if broken_lines != corrupt_info_lines:
+            raise LockBreakMismatch(self, broken_lines, corrupt_info_lines)
+        self.transport.delete(broken_info_path)
+        self.transport.rmdir(tmpname)
+        result = lock.LockResult(self.transport.abspath(self.path))
+        for hook in self.hooks['lock_broken']:
+            hook(result)
 
     def _check_not_locked(self):
         """If the lock is held by this instance, raise an error."""
@@ -387,7 +443,7 @@ class LockDir(object):
         or if the lock has been affected by a bug.
 
         If the lock is not thought to be held, raises LockNotHeld.  If
-        the lock is thought to be held but has been broken, raises 
+        the lock is thought to be held but has been broken, raises
         LockBroken.
         """
         if not self._lock_held:
@@ -399,26 +455,24 @@ class LockDir(object):
         if info.get('nonce') != self.nonce:
             # there is a lock, but not ours
             raise LockBroken(self)
-        
+
     def _read_info_file(self, path):
         """Read one given info file.
 
         peek() reads the info file of the lock holder, if any.
         """
-        return self._parse_info(self.transport.get(path))
+        return self._parse_info(self.transport.get_bytes(path))
 
     def peek(self):
         """Check if the lock is held by anyone.
-        
-        If it is held, this returns the lock info structure as a rio Stanza,
+
+        If it is held, this returns the lock info structure as a dict
         which contains some information about the current lock holder.
         Otherwise returns None.
         """
         try:
             info = self._read_info_file(self._held_info_path)
             self._trace("peek -> held")
-            assert isinstance(info, dict), \
-                    "bad parse result %r" % info
             return info
         except NoSuchFile, e:
             self._trace("peek -> not held")
@@ -426,14 +480,13 @@ class LockDir(object):
     def _prepare_info(self):
         """Write information about a pending lock to a temporary file.
         """
-        import socket
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
         try:
-            user = config.user_email()
-        except errors.NoEmailInUsername:
             user = config.username()
-        s = Stanza(hostname=socket.gethostname(),
+        except errors.NoWhoami:
+            user = osutils.getuser_unicode()
+        s = rio.Stanza(hostname=get_host_name(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),
                    nonce=self.nonce,
@@ -441,12 +494,25 @@ class LockDir(object):
                    )
         return s.to_string()
 
-    def _parse_info(self, info_file):
-        return read_stanza(info_file.readlines()).as_dict()
+    def _parse_info(self, info_bytes):
+        lines = osutils.split_lines(info_bytes)
+        try:
+            stanza = rio.read_stanza(lines)
+        except ValueError, e:
+            mutter('Corrupt lock info file: %r', lines)
+            raise LockCorrupt("could not parse lock info file: " + str(e),
+                              lines)
+        if stanza is None:
+            # see bug 185013; we fairly often end up with the info file being
+            # empty after an interruption; we could log a message here but
+            # there may not be much we can say
+            return {}
+        else:
+            return stanza.as_dict()
 
     def attempt_lock(self):
         """Take the lock; fail if it's already held.
-        
+
         If you wish to block until the lock can be obtained, call wait_lock()
         instead.
 
@@ -455,7 +521,12 @@ class LockDir(object):
         """
         if self._fake_read_lock:
             raise LockContention(self)
-        return self._attempt_lock()
+        result = self._attempt_lock()
+        hook_result = lock.LockResult(self.transport.abspath(self.path),
+                self.nonce)
+        for hook in self.hooks['lock_acquired']:
+            hook(hook_result)
+        return result
 
     def wait_lock(self, timeout=None, poll=None, max_attempts=None):
         """Wait a certain period for a lock.
@@ -468,7 +539,7 @@ class LockDir(object):
 
         :param timeout: Approximate maximum amount of time to wait for the
         lock, in seconds.
-         
+
         :param poll: Delay in seconds between retrying the lock.
 
         :param max_attempts: Maximum number of times to try to lock.
@@ -510,16 +581,27 @@ class LockDir(object):
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
                                                  time.localtime(deadline))
-                self._report_function('%s %s\n'
-                                      '%s\n' # held by
-                                      '%s\n' # locked ... ago
-                                      'Will continue to try until %s\n',
-                                      start,
-                                      formatted_info[0],
-                                      formatted_info[1],
-                                      formatted_info[2],
-                                      deadline_str)
-
+                # As local lock urls are correct we display them.
+                # We avoid displaying remote lock urls.
+                lock_url = self.transport.abspath(self.path)
+                if lock_url.startswith('file://'):
+                    lock_url = lock_url.split('.bzr/')[0]
+                else:
+                    lock_url = ''
+                user, hostname, pid, time_ago = formatted_info
+                msg = ('%s lock %s '        # lock_url
+                    'held by '              # start
+                    '%s\n'                  # user
+                    'at %s '                # hostname
+                    '[process #%s], '       # pid
+                    'acquired %s.')         # time ago
+                msg_args = [start, lock_url, user, hostname, pid, time_ago]
+                if timeout > 0:
+                    msg += ('\nWill continue to try until %s, unless '
+                        'you press Ctrl-C.')
+                    msg_args.append(deadline_str)
+                msg += '\nSee "bzr help break-lock" for more.'
+                self._report_function(msg, *msg_args)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
                 raise LockContention(self)
@@ -527,9 +609,12 @@ class LockDir(object):
                 self._trace("waiting %ss", poll)
                 time.sleep(poll)
             else:
+                # As timeout is always 0 for remote locks
+                # this block is applicable only for local
+                # lock contention
                 self._trace("timeout after waiting %ss", timeout)
-                raise LockContention(self)
-    
+                raise LockContention('(local)', lock_url)
+
     def leave_in_place(self):
         self._locked_via_token = True
 
@@ -538,7 +623,7 @@ class LockDir(object):
 
     def lock_write(self, token=None):
         """Wait for and acquire the lock.
-        
+
         :param token: if this is already locked, then lock_write will fail
             unless the token matches the existing lock.
         :returns: a token if this instance supports tokens, otherwise None.
@@ -550,7 +635,7 @@ class LockDir(object):
         A token should be passed in if you know that you have locked the object
         some other way, and need to synchronise this object's state with that
         fact.
-         
+
         XXX: docstring duplicated from LockableFiles.lock_write.
         """
         if token is not None:
@@ -565,12 +650,12 @@ class LockDir(object):
     def lock_read(self):
         """Compatibility-mode shared lock.
 
-        LockDir doesn't support shared read-only locks, so this 
+        LockDir doesn't support shared read-only locks, so this
         just pretends that the lock is taken but really does nothing.
         """
-        # At the moment Branches are commonly locked for read, but 
+        # At the moment Branches are commonly locked for read, but
         # we can't rely on that remotely.  Once this is cleaned up,
-        # reenable this warning to prevent it coming back in 
+        # reenable this warning to prevent it coming back in
         # -- mbp 20060303
         ## warn("LockDir.lock_read falls back to write lock")
         if self._lock_held or self._fake_read_lock:
@@ -579,12 +664,19 @@ class LockDir(object):
 
     def _format_lock_info(self, info):
         """Turn the contents of peek() into something for the user"""
-        lock_url = self.transport.abspath(self.path)
-        delta = time.time() - int(info['start_time'])
+        start_time = info.get('start_time')
+        if start_time is None:
+            time_ago = '(unknown)'
+        else:
+            time_ago = format_delta(time.time() - int(info['start_time']))
+        user = info.get('user', '<unknown>')
+        hostname = info.get('hostname', '<unknown>')
+        pid = info.get('pid', '<unknown>')
         return [
-            'lock %s' % (lock_url,),
-            'held by %(user)s on host %(hostname)s [process #%(pid)s]' % info,
-            'locked %s' % (format_delta(delta),),
+            user,
+            hostname,
+            pid,
+            time_ago,
             ]
 
     def validate_token(self, token):
