@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,19 +12,19 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 """Black-box tests for bzr aliases.
 """
 
 import os
 
 from bzrlib.branch import Branch
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
 from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
-class TestAliases(ExternalBase):
+class TestAliases(TestCaseWithTransport):
 
     def test_aliases(self):
 
@@ -36,8 +36,8 @@ class TestAliases(ExternalBase):
 
 
         if os.path.isfile(config_filename()):
-            # Something is wrong in environment, 
-            # we risk overwriting users config 
+            # Something is wrong in environment,
+            # we risk overwriting users config
             self.assert_(config_filename() + "exists, abort")
 
         ensure_config_dir_exists()
