@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,19 +12,19 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Black-box tests for bzr revert."""
 
 import os
 
 import bzrlib.osutils
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
 
-class TestRevert(ExternalBase):
+class TestRevert(TestCaseWithTransport):
 
     def _prepare_tree(self):
         self.run_bzr('init')
@@ -46,7 +46,7 @@ class TestRevert(ExternalBase):
         self.assertEquals('modified:\n  dir/file\n', self.run_bzr('status')[0])
 
     def _prepare_rename_mod_tree(self):
-        self.build_tree(['a/', 'a/b', 'a/c', 'a/d/', 'a/d/e', 'f/', 'f/g', 
+        self.build_tree(['a/', 'a/b', 'a/c', 'a/d/', 'a/d/e', 'f/', 'f/g',
                          'f/h', 'f/i'])
         self.run_bzr('init')
         self.run_bzr('add')
@@ -96,7 +96,9 @@ class TestRevert(ExternalBase):
         self.failUnlessExists('a/b')
         self.failUnlessExists('a/d')
         self.failIfExists('a/g')
-        self.failUnlessExists('j')
+        self.expectFailure(
+            "j is in the delta revert applies because j was renamed too",
+            self.failUnlessExists, 'j')
         self.failUnlessExists('h')
         self.run_bzr('revert f')
         self.failIfExists('j')
@@ -154,7 +156,7 @@ class TestRevert(ExternalBase):
                              os.readlink('symlink'))
         else:
             self.log("skipping revert symlink tests")
-        
+
         file('hello', 'wt').write('xyz')
         self.run_bzr('commit -m xyz hello')
         self.run_bzr('revert -r 1 hello')
