@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,26 +12,34 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for LockDir"""
 
 from cStringIO import StringIO
+import os
 from threading import Thread, Lock
 import time
 
 import bzrlib
 from bzrlib import (
     config,
+    errors,
+    lock,
     osutils,
+    tests,
+    transport,
     )
 from bzrlib.errors import (
-        LockBreakMismatch,
-        LockContention, LockError, UnlockableTransport,
-        LockNotHeld, LockBroken
-        )
-from bzrlib.lockdir import LockDir, _DEFAULT_TIMEOUT_SECONDS
-from bzrlib.tests import TestCaseWithTransport
+    LockBreakMismatch,
+    LockBroken,
+    LockContention,
+    LockError,
+    LockFailed,
+    LockNotHeld,
+    )
+from bzrlib.lockdir import LockDir
+from bzrlib.tests import (features, TestCaseWithTransport)
 from bzrlib.trace import note
 
 # These tests sometimes use threads to test the behaviour of lock files with
@@ -101,14 +109,14 @@ class TestLockDir(TestCaseWithTransport):
         """Fail to create lock on readonly transport"""
         t = self.get_readonly_transport()
         lf = LockDir(t, 'test_lock')
-        self.assertRaises(UnlockableTransport, lf.create)
+        self.assertRaises(LockFailed, lf.create)
 
     def test_12_lock_readonly_transport(self):
         """Fail to lock on readonly transport"""
         lf = LockDir(self.get_transport(), 'test_lock')
         lf.create()
         lf = LockDir(self.get_readonly_transport(), 'test_lock')
-        self.assertRaises(UnlockableTransport, lf.attempt_lock)
+        self.assertRaises(LockFailed, lf.attempt_lock)
 
     def test_20_lock_contested(self):
         """Contention to get a lock"""
@@ -118,7 +126,7 @@ class TestLockDir(TestCaseWithTransport):
         lf1.attempt_lock()
         lf2 = LockDir(t, 'test_lock')
         try:
-            # locking is between LockDir instances; aliases within 
+            # locking is between LockDir instances; aliases within
             # a single process are not detected
             lf2.attempt_lock()
             self.fail('Failed to detect lock collision')
@@ -134,6 +142,7 @@ class TestLockDir(TestCaseWithTransport):
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
         lf1.attempt_lock()
+        self.addCleanup(lf1.unlock)
         # lock is held, should get some info on it
         info1 = lf1.peek()
         self.assertEqual(set(info1.keys()),
@@ -153,13 +162,14 @@ class TestLockDir(TestCaseWithTransport):
         lf2 = LockDir(self.get_readonly_transport(), 'test_lock')
         self.assertEqual(lf2.peek(), None)
         lf1.attempt_lock()
+        self.addCleanup(lf1.unlock)
         info2 = lf2.peek()
         self.assertTrue(info2)
         self.assertEqual(info2['nonce'], lf1.nonce)
 
     def test_30_lock_wait_fail(self):
         """Wait on a lock, then fail
-        
+
         We ask to wait up to 400ms; this should fail within at most one
         second.  (Longer times are more realistic but we don't want the test
         suite to take too long, and this should do for now.)
@@ -177,23 +187,25 @@ class TestLockDir(TestCaseWithTransport):
             after = time.time()
             # it should only take about 0.4 seconds, but we allow more time in
             # case the machine is heavily loaded
-            self.assertTrue(after - before <= 8.0, 
+            self.assertTrue(after - before <= 8.0,
                     "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
-        lock_base = lf2.transport.abspath(lf2.path)
         self.assertEqual(1, len(self._logged_reports))
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s\n',
-                         self._logged_reports[0][0])
-        args = self._logged_reports[0][1]
-        self.assertEqual('Unable to obtain', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+        self.assertEqual(self._logged_reports[0][0],
+            '%s lock %s held by %s\n'
+            'at %s [process #%s], acquired %s.\n'
+            'Will continue to try until %s, unless '
+            'you press Ctrl-C.\n'
+            'See "bzr help break-lock" for more.')
+        start, lock_url, user, hostname, pid, time_ago, deadline_str = \
+            self._logged_reports[0][1]
+        self.assertEqual(start, u'Unable to obtain')
+        self.assertEqual(user, u'jrandom@example.com')
+        # skip hostname
+        self.assertContainsRe(pid, r'\d+')
+        self.assertContainsRe(time_ago, r'.* ago')
+        self.assertContainsRe(deadline_str, r'\d{2}:\d{2}:\d{2}')
 
     def test_31_lock_wait_easy(self):
         """Succeed when waiting on a lock with no contention.
@@ -214,9 +226,19 @@ class TestLockDir(TestCaseWithTransport):
     def test_32_lock_wait_succeed(self):
         """Succeed when trying to acquire a lock that gets released
 
-        One thread holds on a lock and then releases it; another 
+        One thread holds on a lock and then releases it; another
         tries to lock it.
         """
+        # This test sometimes fails like this:
+        # Traceback (most recent call last):
+
+        #   File "/home/pqm/bzr-pqm-workdir/home/+trunk/bzrlib/tests/
+        # test_lockdir.py", line 247, in test_32_lock_wait_succeed
+        #     self.assertEqual(1, len(self._logged_reports))
+        # AssertionError: not equal:
+        # a = 1
+        # b = 0
+        raise tests.TestSkipped("Test fails intermittently")
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
@@ -254,35 +276,12 @@ class TestLockDir(TestCaseWithTransport):
         self.assertEndsWith(args[3], ' ago')
         self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
 
-    def test_33_wait(self):
-        """Succeed when waiting on a lock that gets released
-
-        The difference from test_32_lock_wait_succeed is that the second 
-        caller does not actually acquire the lock, but just waits for it
-        to be released.  This is done over a readonly transport.
-        """
-        t = self.get_transport()
-        lf1 = LockDir(t, 'test_lock')
-        lf1.create()
-        lf1.attempt_lock()
-
-        def wait_and_unlock():
-            time.sleep(0.1)
-            lf1.unlock()
-        unlocker = Thread(target=wait_and_unlock)
-        unlocker.start()
-        try:
-            lf2 = LockDir(self.get_readonly_transport(), 'test_lock')
-            before = time.time()
-            # wait but don't lock
-            lf2.wait(timeout=0.4, poll=0.1)
-            after = time.time()
-            self.assertTrue(after - before <= 1.0)
-        finally:
-            unlocker.join()
-
     def test_34_lock_write_waits(self):
-        """LockDir.lock_write() will wait for the lock.""" 
+        """LockDir.lock_write() will wait for the lock."""
+        # the test suite sets the default to 0 to make deadlocks fail fast.
+        # change it for this test, as we want to try a manual deadlock.
+        raise tests.TestSkipped('Timing-sensitive test')
+        bzrlib.lockdir._DEFAULT_TIMEOUT_SECONDS = 300
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
@@ -321,24 +320,27 @@ class TestLockDir(TestCaseWithTransport):
 
     def test_35_wait_lock_changing(self):
         """LockDir.wait_lock() will report if the lock changes underneath.
-        
+
         This is the stages we want to happen:
 
         0) Synchronization locks are created and locked.
         1) Lock1 obtains the lockdir, and releases the 'check' lock.
         2) Lock2 grabs the 'check' lock, and checks the lockdir.
-           It sees the lockdir is already acquired, reports the fact, 
+           It sees the lockdir is already acquired, reports the fact,
            and unsets the 'checked' lock.
         3) Thread1 blocks on acquiring the 'checked' lock, and then tells
            Lock1 to release and acquire the lockdir. This resets the 'check'
            lock.
         4) Lock2 acquires the 'check' lock, and checks again. It notices
-           that the holder of the lock has changed, and so reports a new 
+           that the holder of the lock has changed, and so reports a new
            lock holder.
         5) Thread1 blocks on the 'checked' lock, this time, it completely
            unlocks the lockdir, allowing Lock2 to acquire the lock.
         """
 
+        raise tests.KnownFailure(
+            "timing dependency in lock tests (#213182)")
+
         wait_to_check_lock = Lock()
         wait_until_checked_lock = Lock()
 
@@ -404,7 +406,7 @@ class TestLockDir(TestCaseWithTransport):
         unlocker.start()
         try:
             # Wait and play against the other thread
-            lf2.wait_lock(timeout=1.0, poll=0.01)
+            lf2.wait_lock(timeout=20.0, poll=0.01)
         finally:
             unlocker.join()
         lf2.unlock()
@@ -412,10 +414,12 @@ class TestLockDir(TestCaseWithTransport):
         # There should be 2 reports, because the lock changed
         lock_base = lf2.transport.abspath(lf2.path)
         self.assertEqual(2, len(self._logged_reports))
-
+        lock_url = lf2.transport.abspath(lf2.path)
         self.assertEqual('%s %s\n'
                          '%s\n%s\n'
-                         'Will continue to try until %s\n',
+                         'Will continue to try until %s, unless '
+                         'you press Ctrl-C.\n'
+                         'See "bzr help break-lock" for more.',
                          self._logged_reports[0][0])
         args = self._logged_reports[0][1]
         self.assertEqual('Unable to obtain', args[0])
@@ -427,7 +431,9 @@ class TestLockDir(TestCaseWithTransport):
 
         self.assertEqual('%s %s\n'
                          '%s\n%s\n'
-                         'Will continue to try until %s\n',
+                         'Will continue to try until %s, unless '
+                         'you press Ctrl-C.\n'
+                         'See "bzr help break-lock" for more.',
                          self._logged_reports[1][0])
         args = self._logged_reports[1][1]
         self.assertEqual('Lock owner changed for', args[0])
@@ -443,6 +449,7 @@ class TestLockDir(TestCaseWithTransport):
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
         lf1.attempt_lock()
+        self.addCleanup(lf1.unlock)
         lf1.confirm()
 
     def test_41_confirm_not_held(self):
@@ -460,6 +467,9 @@ class TestLockDir(TestCaseWithTransport):
         lf1.attempt_lock()
         t.move('test_lock', 'lock_gone_now')
         self.assertRaises(LockBroken, lf1.confirm)
+        # Clean up
+        t.move('lock_gone_now', 'test_lock')
+        lf1.unlock()
 
     def test_43_break(self):
         """Break a lock whose caller has forgotten it"""
@@ -476,6 +486,7 @@ class TestLockDir(TestCaseWithTransport):
         lf2.force_break(holder_info)
         # now we should be able to take it
         lf2.attempt_lock()
+        self.addCleanup(lf2.unlock)
         lf2.confirm()
 
     def test_44_break_already_released(self):
@@ -493,6 +504,7 @@ class TestLockDir(TestCaseWithTransport):
         lf2.force_break(holder_info)
         # now we should be able to take it
         lf2.attempt_lock()
+        self.addCleanup(lf2.unlock)
         lf2.confirm()
 
     def test_45_break_mismatch(self):
@@ -524,7 +536,7 @@ class TestLockDir(TestCaseWithTransport):
         """Check the on-disk representation of LockDirs is as expected.
 
         There should always be a top-level directory named by the lock.
-        When the lock is held, there should be a lockname/held directory 
+        When the lock is held, there should be a lockname/held directory
         containing an info file.
         """
         t = self.get_transport()
@@ -545,15 +557,69 @@ class TestLockDir(TestCaseWithTransport):
         # do this without IO redirection to ensure it doesn't prompt.
         self.assertRaises(AssertionError, ld1.break_lock)
         orig_factory = bzrlib.ui.ui_factory
-        # silent ui - no need for stdout
-        bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
-        bzrlib.ui.ui_factory.stdin = StringIO("y\n")
+        bzrlib.ui.ui_factory = bzrlib.ui.CannedInputUIFactory([True])
         try:
             ld2.break_lock()
             self.assertRaises(LockBroken, ld1.unlock)
         finally:
             bzrlib.ui.ui_factory = orig_factory
 
+    def test_break_lock_corrupt_info(self):
+        """break_lock works even if the info file is corrupt (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.put_bytes_non_atomic('test_lock/held/info', '\0')
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+        ui = LoggingUIFactory()
+        orig_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = ui
+        try:
+            ld2.break_lock()
+            self.assertLength(1, ui.prompts)
+            self.assertEqual('boolean', ui.prompts[0][0])
+            self.assertStartsWith(ui.prompts[0][1], 'Break (corrupt LockDir')
+            self.assertRaises(LockBroken, ld.unlock)
+        finally:
+            bzrlib.ui.ui_factory = orig_factory
+
+    def test_break_lock_missing_info(self):
+        """break_lock works even if the info file is missing (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.delete('test_lock/held/info')
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+        ui = LoggingUIFactory()
+        orig_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = ui
+        try:
+            ld2.break_lock()
+            self.assertRaises(LockBroken, ld.unlock)
+            self.assertLength(0, ui.prompts)
+        finally:
+            bzrlib.ui.ui_factory = orig_factory
+        # Suppress warnings due to ld not being unlocked
+        # XXX: if lock_broken hook was invoked in this case, this hack would
+        # not be necessary.  - Andrew Bennetts, 2010-09-06.
+        del self._lock_actions[:]
+
     def test_create_missing_base_directory(self):
         """If LockDir.path doesn't exist, it can be created
 
@@ -586,11 +652,10 @@ class TestLockDir(TestCaseWithTransport):
             info_list = ld1._format_lock_info(ld1.peek())
         finally:
             ld1.unlock()
-        self.assertEqual('lock %s' % (ld1.transport.abspath(ld1.path),),
-                         info_list[0])
-        self.assertContainsRe(info_list[1],
-                              r'^held by .* on host .* \[process #\d*\]$')
-        self.assertContainsRe(info_list[2], r'locked \d+ seconds? ago$')
+        self.assertEqual(info_list[0], u'jrandom@example.com')
+        # info_list[1] is hostname. we skip this.
+        self.assertContainsRe(info_list[2], '^\d+$') # pid
+        self.assertContainsRe(info_list[3], r'^\d+ seconds? ago$') # time_ago
 
     def test_lock_without_email(self):
         global_config = config.GlobalConfig()
@@ -600,3 +665,212 @@ class TestLockDir(TestCaseWithTransport):
         ld1.create()
         ld1.lock_write()
         ld1.unlock()
+
+    def test_lock_permission(self):
+        self.requireFeature(features.not_running_as_root)
+        if not osutils.supports_posix_readonly():
+            raise tests.TestSkipped('Cannot induce a permission failure')
+        ld1 = self.get_lock()
+        lock_path = ld1.transport.local_abspath('test_lock')
+        os.mkdir(lock_path)
+        osutils.make_readonly(lock_path)
+        self.assertRaises(errors.LockFailed, ld1.attempt_lock)
+
+    def test_lock_by_token(self):
+        ld1 = self.get_lock()
+        token = ld1.lock_write()
+        self.addCleanup(ld1.unlock)
+        self.assertNotEqual(None, token)
+        ld2 = self.get_lock()
+        t2 = ld2.lock_write(token)
+        self.addCleanup(ld2.unlock)
+        self.assertEqual(token, t2)
+
+    def test_lock_with_buggy_rename(self):
+        # test that lock acquisition handles servers which pretend they
+        # renamed correctly but that actually fail
+        t = transport.get_transport('brokenrename+' + self.get_url())
+        ld1 = LockDir(t, 'test_lock')
+        ld1.create()
+        ld1.attempt_lock()
+        ld2 = LockDir(t, 'test_lock')
+        # we should fail to lock
+        e = self.assertRaises(errors.LockContention, ld2.attempt_lock)
+        # now the original caller should succeed in unlocking
+        ld1.unlock()
+        # and there should be nothing left over
+        self.assertEquals([], t.list_dir('test_lock'))
+
+    def test_failed_lock_leaves_no_trash(self):
+        # if we fail to acquire the lock, we don't leave pending directories
+        # behind -- https://bugs.launchpad.net/bzr/+bug/109169
+        ld1 = self.get_lock()
+        ld2 = self.get_lock()
+        # should be nothing before we start
+        ld1.create()
+        t = self.get_transport().clone('test_lock')
+        def check_dir(a):
+            self.assertEquals(a, t.list_dir('.'))
+        check_dir([])
+        # when held, that's all we see
+        ld1.attempt_lock()
+        self.addCleanup(ld1.unlock)
+        check_dir(['held'])
+        # second guy should fail
+        self.assertRaises(errors.LockContention, ld2.attempt_lock)
+        # no kibble
+        check_dir(['held'])
+
+    def test_no_lockdir_info(self):
+        """We can cope with empty info files."""
+        # This seems like a fairly common failure case - see
+        # <https://bugs.launchpad.net/bzr/+bug/185103> and all its dupes.
+        # Processes are often interrupted after opening the file
+        # before the actual contents are committed.
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        t.put_bytes('test_lock/held/info', '')
+        lf = LockDir(t, 'test_lock')
+        info = lf.peek()
+        formatted_info = lf._format_lock_info(info)
+        self.assertEquals(
+            ['<unknown>', '<unknown>', '<unknown>', '(unknown)'],
+            formatted_info)
+
+    def test_corrupt_lockdir_info(self):
+        """We can cope with corrupt (and thus unparseable) info files."""
+        # This seems like a fairly common failure case too - see
+        # <https://bugs.edge.launchpad.net/bzr/+bug/619872> for instance.
+        # In particular some systems tend to fill recently created files with
+        # nul bytes after recovering from a system crash.
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        t.put_bytes('test_lock/held/info', '\0')
+        lf = LockDir(t, 'test_lock')
+        self.assertRaises(errors.LockCorrupt, lf.peek)
+        # Currently attempt_lock gives LockContention, but LockCorrupt would be
+        # a reasonable result too.
+        self.assertRaises(
+            (errors.LockCorrupt, errors.LockContention), lf.attempt_lock)
+        self.assertRaises(errors.LockCorrupt, lf.validate_token, 'fake token')
+
+    def test_missing_lockdir_info(self):
+        """We can cope with absent info files."""
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        lf = LockDir(t, 'test_lock')
+        # In this case we expect the 'not held' result from peek, because peek
+        # cannot be expected to notice that there is a 'held' directory with no
+        # 'info' file.
+        self.assertEqual(None, lf.peek())
+        # And lock/unlock may work or give LockContention (but not any other
+        # error).
+        try:
+            lf.attempt_lock()
+        except LockContention:
+            # LockContention is ok, and expected on Windows
+            pass
+        else:
+            # no error is ok, and expected on POSIX (because POSIX allows
+            # os.rename over an empty directory).
+            lf.unlock()
+        # Currently raises TokenMismatch, but LockCorrupt would be reasonable
+        # too.
+        self.assertRaises(
+            (errors.TokenMismatch, errors.LockCorrupt),
+            lf.validate_token, 'fake token')
+
+
+class TestLockDirHooks(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestLockDirHooks, self).setUp()
+        self._calls = []
+
+    def get_lock(self):
+        return LockDir(self.get_transport(), 'test_lock')
+
+    def record_hook(self, result):
+        self._calls.append(result)
+
+    def test_LockDir_acquired_success(self):
+        # the LockDir.lock_acquired hook fires when a lock is acquired.
+        LockDir.hooks.install_named_hook('lock_acquired',
+                                         self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+        ld.unlock()
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_acquired_fail(self):
+        # the LockDir.lock_acquired hook does not fire on failure.
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld2.attempt_lock()
+        # install a lock hook now, when the disk lock is locked
+        LockDir.hooks.install_named_hook('lock_acquired',
+                                         self.record_hook, 'record_hook')
+        self.assertRaises(errors.LockContention, ld.attempt_lock)
+        self.assertEqual([], self._calls)
+        ld2.unlock()
+        self.assertEqual([], self._calls)
+
+    def test_LockDir_released_success(self):
+        # the LockDir.lock_released hook fires when a lock is acquired.
+        LockDir.hooks.install_named_hook('lock_released',
+                                         self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        self.assertEqual([], self._calls)
+        ld.unlock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_released_fail(self):
+        # the LockDir.lock_released hook does not fire on failure.
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld.attempt_lock()
+        ld2.force_break(ld2.peek())
+        LockDir.hooks.install_named_hook('lock_released',
+                                         self.record_hook, 'record_hook')
+        self.assertRaises(LockBroken, ld.unlock)
+        self.assertEqual([], self._calls)
+
+    def test_LockDir_broken_success(self):
+        # the LockDir.lock_broken hook fires when a lock is broken.
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        result = ld.attempt_lock()
+        LockDir.hooks.install_named_hook('lock_broken',
+                                         self.record_hook, 'record_hook')
+        ld2.force_break(ld2.peek())
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_broken_failure(self):
+        # the LockDir.lock_broken hook does not fires when a lock is already
+        # released.
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        result = ld.attempt_lock()
+        holder_info = ld2.peek()
+        ld.unlock()
+        LockDir.hooks.install_named_hook('lock_broken',
+                                         self.record_hook, 'record_hook')
+        ld2.force_break(holder_info)
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([], self._calls)
