@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,29 +12,29 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
-import stat
 import sys
 
 import bzrlib
 from bzrlib import (
+    errors,
     generate_ids,
+    merge_directive,
     osutils,
     )
-from bzrlib.add import smart_add_tree
-from bzrlib.builtins import merge
 from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
-from bzrlib.errors import (NotBranchError, NotVersionedError,
-                           WorkingTreeNotRevision, BzrCommandError, NoDiff3)
-import bzrlib.inventory as inventory
-from bzrlib.merge import Merge3Merger, Diff3Merger, WeaveMerger
-from bzrlib.osutils import (file_kind, getcwd, pathjoin, rename,
-                            sha_file,
-                            )
+from bzrlib.merge import (
+    Merge3Merger,
+    Diff3Merger,
+    WeaveMerger,
+    Merger,
+    )
+from bzrlib.osutils import getcwd, pathjoin
+from bzrlib import progress
 from bzrlib.transform import TreeTransform
-from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree
 
 
@@ -49,6 +49,7 @@ class MergeBuilder(object):
            # the tests perform pulls, so need a branch that is writeable.
            wt.lock_write()
            wt.set_root_id(self.tree_root)
+           wt.flush()
            tt = TreeTransform(wt)
            return wt, tt
         self.base, self.base_tt = wt('base')
@@ -57,11 +58,12 @@ class MergeBuilder(object):
 
     def get_cset_path(self, parent, name):
         if name is None:
-            assert (parent is None)
+            if parent is not None:
+                raise AssertionError()
             return None
         return pathjoin(self.cset.entries[parent].path, name)
 
-    def add_file(self, id, parent, name, contents, executable, this=True, 
+    def add_file(self, id, parent, name, contents, executable, this=True,
                  base=True, other=True):
         def new_file(tt):
             parent_id = tt.trans_id_file_id(parent)
@@ -71,26 +73,47 @@ class MergeBuilder(object):
                 new_file(tt)
 
     def merge(self, merge_type=Merge3Merger, interesting_ids=None, **kwargs):
+        merger = self.make_merger(merge_type, interesting_ids, **kwargs)
+        merger.do_merge()
+        return merger.cooked_conflicts
+
+    def make_preview_transform(self):
+        merger = self.make_merger(Merge3Merger, None, this_revision_tree=True)
+        return merger.make_preview_transform()
+
+    def make_merger(self, merge_type, interesting_ids,
+            this_revision_tree=False, **kwargs):
         self.base_tt.apply()
         self.base.commit('base commit')
         for tt, wt in ((self.this_tt, self.this), (self.other_tt, self.other)):
             # why does this not do wt.pull() ?
             wt.branch.pull(self.base.branch)
             wt.set_parent_ids([wt.branch.last_revision()])
+            wt.flush()
+            # We maintain a write lock, so make sure changes are flushed to
+            # disk first
             tt.apply()
             wt.commit('branch commit')
-            assert len(wt.branch.revision_history()) == 2
+            wt.flush()
+            if len(wt.branch.revision_history()) != 2:
+                raise AssertionError()
         self.this.branch.fetch(self.other.branch)
         other_basis = self.other.branch.basis_tree()
-        merger = merge_type(self.this, self.this, self.base, other_basis, 
-                            interesting_ids=interesting_ids, **kwargs)
-        return merger.cooked_conflicts
+        if this_revision_tree:
+            self.this.commit('message')
+            this_tree = self.this.basis_tree()
+        else:
+            this_tree = self.this
+        merger = merge_type(this_tree, self.this, self.base, other_basis,
+                            interesting_ids=interesting_ids, do_merge=False,
+                            this_branch=self.this.branch, **kwargs)
+        return merger
 
     def list_transforms(self):
         return [self.this_tt, self.base_tt, self.other_tt]
 
     def selected_transforms(self, this, base, other):
-        pairs = [(this, self.this_tt), (base, self.base_tt), 
+        pairs = [(this, self.this_tt), (base, self.base_tt),
                  (other, self.other_tt)]
         return [(v, tt) for (v, tt) in pairs if v is not None]
 
@@ -107,13 +130,14 @@ class MergeBuilder(object):
                 tt.cancel_versioning(trans_id)
                 tt.set_executability(None, trans_id)
 
-    def add_dir(self, file_id, parent, name):
-        for tt in self.list_transforms():
-            parent_id = tt.trans_id_file_id(parent)
-            tt.new_directory(name, parent_id, file_id)
+    def add_dir(self, file_id, parent, name, this=True, base=True, other=True):
+        for option, tt in self.selected_transforms(this, base, other):
+            if option is True:
+                parent_id = tt.trans_id_file_id(parent)
+                tt.new_directory(name, parent_id, file_id)
 
     def change_name(self, id, base=None, this=None, other=None):
-        for val, tt in ((base, self.base_tt), (this, self.this_tt), 
+        for val, tt in ((base, self.base_tt), (this, self.this_tt),
                         (other, self.other_tt)):
             if val is None:
                 continue
@@ -162,7 +186,7 @@ class MergeBuilder(object):
             if parent_dir == "":
                 return None
             return orig_inventory_by_path[parent_dir]
-        
+
         def new_path(file_id):
             if fild_id in inventory_change:
                 return inventory_change[file_id]
@@ -226,7 +250,7 @@ class MergeTest(TestCaseWithTransport):
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
         self.assertEqual(builder.this.get_file("2").read(), "hello1" )
         builder.cleanup()
-        
+
     def test_file_moves(self):
         """Test moves"""
         builder = MergeBuilder(getcwd())
@@ -259,9 +283,12 @@ class MergeTest(TestCaseWithTransport):
 
     def test_contents_merge2(self):
         """Test diff3 merging"""
+        if sys.platform == 'win32':
+            raise TestSkipped("diff3 does not have --binary flag"
+                              " and therefore always fails on win32")
         try:
             self.do_contents_test(Diff3Merger)
-        except NoDiff3:
+        except errors.NoDiff3:
             raise TestSkipped("diff3 not available")
 
     def test_contents_merge3(self):
@@ -307,14 +334,14 @@ y
         builder.remove_file("4", base=True)
         builder.add_file("5", builder.tree_root, "name7", "a\nb\nc\nd\ne\nf\n",
                          True)
-        builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n", 
+        builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n",
                                      this="a\nb\nc\nd\ne\nz\n")
         conflicts = builder.merge(merge_factory)
         try:
             self.assertEqual([], conflicts)
             self.assertEqual("text4", builder.this.get_file("1").read())
             self.assertEqual("text2", builder.this.get_file("2").read())
-            self.assertEqual("a\nz\nc\nd\ne\nz\n", 
+            self.assertEqual("a\nz\nc\nd\ne\nz\n",
                              builder.this.get_file("5").read())
             self.assertTrue(builder.this.is_executable("1"))
             self.assertFalse(builder.this.is_executable("2"))
@@ -347,7 +374,7 @@ y
             builder.add_symlink("2", builder.tree_root, "name2", "target1")
             builder.change_target("2", other="target4", base="text3")
             conflicts = builder.merge()
-            self.assertEqual(conflicts, [ContentsConflict('name2', 
+            self.assertEqual(conflicts, [ContentsConflict('name2',
                                                           file_id='2')])
             builder.cleanup()
 
@@ -403,22 +430,31 @@ y
         builder = MergeBuilder(getcwd())
         builder.add_file("1", builder.tree_root, "name1", "text1", False)
         builder.remove_file("1", other=True)
-        builder.add_file("2", builder.tree_root, "name1", "text1", False, 
+        builder.add_file("2", builder.tree_root, "name1", "text1", False,
                          this=False, base=False)
         conflicts = builder.merge()
-        self.assertEqual(conflicts, []) 
+        self.assertEqual(conflicts, [])
+        builder.cleanup()
+
+    def test_merge_one_renamed(self):
+        builder = MergeBuilder(getcwd())
+        builder.add_file('1', builder.tree_root, 'name1', 'text1a', False)
+        builder.change_name('1', this='name2')
+        builder.change_contents('1', other='text2')
+        builder.merge(interesting_files=['name2'])
+        self.assertEqual('text2', builder.this.get_file('1').read())
         builder.cleanup()
 
 
 class FunctionalMergeTest(TestCaseWithTransport):
 
     def test_trivial_star_merge(self):
-        """Test that merges in a star shape Just Work.""" 
+        """Test that merges in a star shape Just Work."""
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
         tree = self.make_branch_and_tree('original')
         branch = tree.branch
-        smart_add_tree(tree, ["original"])
+        tree.smart_add(["original"])
         tree.commit("start branch.", verbose=False)
         # Mary branches it.
         self.build_tree(("mary/",))
@@ -436,45 +472,68 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file.close()
         mary_tree.commit("change file2")
         # john should be able to merge with no conflicts.
-        merge_type = Merge3Merger
         base = [None, None]
         other = ("mary", -1)
-        self.assertRaises(BzrCommandError, merge, other, base, check_clean=True,
-                          merge_type=WeaveMerger, this_dir="original",
-                          show_base=True)
-        merge(other, base, check_clean=True, merge_type=merge_type,
-              this_dir="original")
+        tree.merge_from_branch(mary_tree.branch)
         self.assertEqual("John\n", open("original/file1", "rt").read())
         self.assertEqual("Mary\n", open("original/file2", "rt").read())
- 
+
     def test_conflicts(self):
-        os.mkdir('a')
         wta = self.make_branch_and_tree('a')
-        a = wta.branch
-        file('a/file', 'wb').write('contents\n')
+        self.build_tree_contents([('a/file', 'contents\n')])
         wta.add('file')
         wta.commit('base revision', allow_pointless=False)
-        d_b = a.bzrdir.clone('b')
-        b = d_b.open_branch()
-        file('a/file', 'wb').write('other contents\n')
+        d_b = wta.branch.bzrdir.clone('b')
+        self.build_tree_contents([('a/file', 'other contents\n')])
         wta.commit('other revision', allow_pointless=False)
-        file('b/file', 'wb').write('this contents contents\n')
+        self.build_tree_contents([('b/file', 'this contents contents\n')])
         wtb = d_b.open_workingtree()
         wtb.commit('this revision', allow_pointless=False)
-        self.assertEqual(merge(['a', -1], [None, None], this_dir='b'), 1)
-        self.assert_(os.path.lexists('b/file.THIS'))
-        self.assert_(os.path.lexists('b/file.BASE'))
-        self.assert_(os.path.lexists('b/file.OTHER'))
-        self.assertRaises(WorkingTreeNotRevision, merge, ['a', -1], 
-                          [None, None], this_dir='b', check_clean=False,
-                          merge_type=WeaveMerger)
-        wtb.revert([])
-        self.assertEqual(merge(['a', -1], [None, None], this_dir='b', 
-                               check_clean=False, merge_type=WeaveMerger), 1)
-        self.assert_(os.path.lexists('b/file'))
-        self.assert_(os.path.lexists('b/file.THIS'))
-        self.assert_(not os.path.lexists('b/file.BASE'))
-        self.assert_(os.path.lexists('b/file.OTHER'))
+        self.assertEqual(1, wtb.merge_from_branch(wta.branch))
+        self.failUnlessExists('b/file.THIS')
+        self.failUnlessExists('b/file.BASE')
+        self.failUnlessExists('b/file.OTHER')
+        wtb.revert()
+        self.assertEqual(1, wtb.merge_from_branch(wta.branch,
+                                                  merge_type=WeaveMerger))
+        self.failUnlessExists('b/file')
+        self.failUnlessExists('b/file.THIS')
+        self.failUnlessExists('b/file.BASE')
+        self.failUnlessExists('b/file.OTHER')
+
+    def test_weave_conflicts_not_in_base(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        # See bug #494197
+        #  A        base revision (before criss-cross)
+        #  |\
+        #  B C      B does nothing, C adds 'foo'
+        #  |X|
+        #  D E      D and E modify foo in incompatible ways
+        #
+        # Merging will conflict, with C as a clean base text. However, the
+        # current code uses A as the global base and 'foo' doesn't exist there.
+        # It isn't trivial to create foo.BASE because it tries to look up
+        # attributes like 'executable' in A.
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'TREE_ROOT', 'directory', None))])
+        builder.build_snapshot('B-id', ['A-id'], [])
+        builder.build_snapshot('C-id', ['A-id'], [
+            ('add', ('foo', 'foo-id', 'file', 'orig\ncontents\n'))])
+        builder.build_snapshot('D-id', ['B-id', 'C-id'], [
+            ('add', ('foo', 'foo-id', 'file', 'orig\ncontents\nand D\n'))])
+        builder.build_snapshot('E-id', ['C-id', 'B-id'], [
+            ('modify', ('foo-id', 'orig\ncontents\nand E\n'))])
+        builder.finish_series()
+        tree = builder.get_branch().create_checkout('tree', lightweight=True)
+        self.assertEqual(1, tree.merge_from_branch(tree.branch,
+                                                   to_revision='D-id',
+                                                   merge_type=WeaveMerger))
+        self.failUnlessExists('tree/foo.THIS')
+        self.failUnlessExists('tree/foo.OTHER')
+        self.expectFailure('fail to create .BASE in some criss-cross merges',
+            self.failUnlessExists, 'tree/foo.BASE')
+        self.failUnlessExists('tree/foo.BASE')
 
     def test_merge_unrelated(self):
         """Sucessfully merges unrelated branches with no common names"""
@@ -488,7 +547,7 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('b/b_file', 'wb').write('contents\n')
         wtb.add('b_file')
         b_rev = wtb.commit('b_revision', allow_pointless=False)
-        merge(['b', -1], ['b', 0], this_dir='a')
+        wta.merge_from_branch(wtb.branch, b_rev, 'null:')
         self.assert_(os.path.lexists('a/b_file'))
         self.assertEqual([b_rev], wta.get_parent_ids()[1:])
 
@@ -504,7 +563,7 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('b/file', 'wb').write('contents\n')
         wtb.add('file')
         b_rev = wtb.commit('b_revision', allow_pointless=False)
-        merge(['b', -1], ['b', 0], this_dir='a')
+        wta.merge_from_branch(wtb.branch, b_rev, 'null:')
         self.assert_(os.path.lexists('a/file'))
         self.assert_(os.path.lexists('a/file.moved'))
         self.assertEqual([b_rev], wta.get_parent_ids()[1:])
@@ -514,13 +573,14 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('a/file', 'wb').write('contents\n')
         wta.add('file')
         wta.commit('a_revision', allow_pointless=False)
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         os.remove('a/file')
         wta.commit('removed file', allow_pointless=False)
         file('b/file', 'wb').write('changed contents\n')
         wtb = WorkingTree.open('b')
         wtb.commit('changed file', allow_pointless=False)
-        merge(['a', -1], ['a', 1], this_dir='b')
+        wtb.merge_from_branch(wta.branch, wta.branch.last_revision(),
+                              wta.branch.get_rev_id(1))
         self.failIf(os.path.lexists('b/file'))
 
     def test_merge_metadata_vs_deletion(self):
@@ -529,7 +589,7 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('a/file', 'wb').write('contents\n')
         a_wt.add('file')
         a_wt.commit('r0')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         os.chmod('b/file', 0755)
         os.remove('a/file')
@@ -537,23 +597,24 @@ class FunctionalMergeTest(TestCaseWithTransport):
         self.assertEqual(a_wt.branch.revno(), 2)
         self.assertFalse(os.path.exists('a/file'))
         b_wt.commit('exec a')
-        merge(['b', -1], ['b', 0], this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.last_revision(), 'null:')
         self.assert_(os.path.exists('a/file'))
 
     def test_merge_swapping_renames(self):
         a_wt = self.make_branch_and_tree('a')
         file('a/un','wb').write('UN')
         file('a/deux','wb').write('DEUX')
-        a_wt.add('un', 'un')
-        a_wt.add('deux', 'deux')
+        a_wt.add('un', 'un-id')
+        a_wt.add('deux', 'deux-id')
         a_wt.commit('r0', rev_id='r0')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         b_wt.rename_one('un','tmp')
         b_wt.rename_one('deux','un')
         b_wt.rename_one('tmp','deux')
         b_wt.commit('r1', rev_id='r1')
-        self.assertEqual(0, merge(['b', -1], ['b', 1], this_dir='a'))
+        self.assertEqual(0, a_wt.merge_from_branch(b_wt.branch,
+            b_wt.branch.last_revision(), b_wt.branch.get_rev_id(1)))
         self.failUnlessExists('a/un')
         self.failUnless('a/deux')
         self.assertFalse(os.path.exists('a/tmp'))
@@ -565,20 +626,21 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('a/file', 'wb').write('THIS')
         a_wt.add('file')
         a_wt.commit('r0')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         os.remove('b/file')
         b_wt.commit('r1')
         file('b/file', 'wb').write('THAT')
         b_wt.add('file')
         b_wt.commit('r2')
-        merge(['b', -1],['b', 1],this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.branch.last_revision(),
+                               b_wt.branch.get_rev_id(1))
         self.assert_(os.path.exists('a/file'))
         self.assertEqual(file('a/file').read(),'THAT')
 
     def test_merge_rename_before_create(self):
         """rename before create
-        
+
         This case requires that you must not do creates
         before move-into-place:
 
@@ -594,13 +656,14 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('a/foo', 'wb').write('A/FOO')
         a_wt.add('foo')
         a_wt.commit('added foo')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         b_wt.rename_one('foo', 'bar')
         file('b/foo', 'wb').write('B/FOO')
         b_wt.add('foo')
         b_wt.commit('moved foo to bar, added new foo')
-        merge(['b', -1],['b', 1],this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.branch.last_revision(),
+                               b_wt.branch.get_rev_id(1))
 
     def test_merge_create_before_rename(self):
         """create before rename, target parents before children
@@ -622,13 +685,14 @@ class FunctionalMergeTest(TestCaseWithTransport):
         file('a/foo', 'wb').write('A/FOO')
         a_wt.add('foo')
         a_wt.commit('added foo')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         os.mkdir('b/bar')
         b_wt.add('bar')
         b_wt.rename_one('foo', 'bar/foo')
         b_wt.commit('created bar dir, moved foo into bar')
-        merge(['b', -1],['b', 1],this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.branch.last_revision(),
+                               b_wt.branch.get_rev_id(1))
 
     def test_merge_rename_to_temp_before_delete(self):
         """rename to temp before delete, source children before parents
@@ -636,7 +700,7 @@ class FunctionalMergeTest(TestCaseWithTransport):
         This case requires that you must not do deletes before
         move-out-of-the-way, and that you must not do children
         after parents:
-        
+
         $ mkdir foo
         $ touch foo/bar
         $ bzr add foo/bar
@@ -651,20 +715,21 @@ class FunctionalMergeTest(TestCaseWithTransport):
         a_wt.add('foo')
         a_wt.add('foo/bar')
         a_wt.commit('added foo/bar')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         b_wt.rename_one('foo/bar', 'bar')
         os.rmdir('b/foo')
         b_wt.remove('foo')
         b_wt.commit('moved foo/bar to bar, deleted foo')
-        merge(['b', -1],['b', 1],this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.branch.last_revision(),
+                               b_wt.branch.get_rev_id(1))
 
     def test_merge_delete_before_rename_to_temp(self):
         """delete before rename to temp
 
         This case requires that you must not do
         move-out-of-the-way before deletes:
-        
+
         $ touch foo
         $ touch bar
         $ bzr add foo bar
@@ -680,11 +745,80 @@ class FunctionalMergeTest(TestCaseWithTransport):
         a_wt.add('foo')
         a_wt.add('bar')
         a_wt.commit('added foo and bar')
-        self.run_bzr('branch', 'a', 'b')
+        self.run_bzr('branch a b')
         b_wt = WorkingTree.open('b')
         os.unlink('b/foo')
         b_wt.remove('foo')
         b_wt.rename_one('bar', 'foo')
         b_wt.commit('deleted foo, renamed bar to foo')
-        merge(['b', -1],['b', 1],this_dir='a')
+        a_wt.merge_from_branch(b_wt.branch, b_wt.branch.last_revision(),
+                               b_wt.branch.get_rev_id(1))
 
+
+class TestMerger(TestCaseWithTransport):
+
+    def set_up_trees(self):
+        this = self.make_branch_and_tree('this')
+        this.commit('rev1', rev_id='rev1')
+        other = this.bzrdir.sprout('other').open_workingtree()
+        this.commit('rev2a', rev_id='rev2a')
+        other.commit('rev2b', rev_id='rev2b')
+        return this, other
+
+    def test_from_revision_ids(self):
+        this, other = self.set_up_trees()
+        self.assertRaises(errors.NoSuchRevision, Merger.from_revision_ids,
+                          None, this, 'rev2b')
+        this.lock_write()
+        self.addCleanup(this.unlock)
+        merger = Merger.from_revision_ids(None, this,
+            'rev2b', other_branch=other.branch)
+        self.assertEqual('rev2b', merger.other_rev_id)
+        self.assertEqual('rev1', merger.base_rev_id)
+        merger = Merger.from_revision_ids(None, this,
+            'rev2b', 'rev2a', other_branch=other.branch)
+        self.assertEqual('rev2a', merger.base_rev_id)
+
+    def test_from_uncommitted(self):
+        this, other = self.set_up_trees()
+        merger = Merger.from_uncommitted(this, other, None)
+        self.assertIs(other, merger.other_tree)
+        self.assertIs(None, merger.other_rev_id)
+        self.assertEqual('rev2b', merger.base_rev_id)
+
+    def prepare_for_merging(self):
+        this, other = self.set_up_trees()
+        other.commit('rev3', rev_id='rev3')
+        this.lock_write()
+        self.addCleanup(this.unlock)
+        return this, other
+
+    def test_from_mergeable(self):
+        this, other = self.prepare_for_merging()
+        md = merge_directive.MergeDirective2.from_objects(
+            other.branch.repository, 'rev3', 0, 0, 'this')
+        other.lock_read()
+        self.addCleanup(other.unlock)
+        merger, verified = Merger.from_mergeable(this, md,
+            None)
+        md.patch = None
+        merger, verified = Merger.from_mergeable(this, md,
+            None)
+        self.assertEqual('inapplicable', verified)
+        self.assertEqual('rev3', merger.other_rev_id)
+        self.assertEqual('rev1', merger.base_rev_id)
+        md.base_revision_id = 'rev2b'
+        merger, verified = Merger.from_mergeable(this, md,
+            None)
+        self.assertEqual('rev2b', merger.base_rev_id)
+
+    def test_from_mergeable_old_merge_directive(self):
+        this, other = self.prepare_for_merging()
+        other.lock_write()
+        self.addCleanup(other.unlock)
+        md = merge_directive.MergeDirective.from_objects(
+            other.branch.repository, 'rev3', 0, 0, 'this')
+        merger, verified = Merger.from_mergeable(this, md,
+            None)
+        self.assertEqual('rev3', merger.other_rev_id)
+        self.assertEqual('rev1', merger.base_rev_id)
