@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Test commit message editor.
 """
@@ -20,11 +20,13 @@
 import os
 import sys
 
-import bzrlib
 from bzrlib import (
+    commit,
     errors,
     msgeditor,
     osutils,
+    tests,
+    trace,
     )
 from bzrlib.branch import Branch
 from bzrlib.config import ensure_config_dir_exists, config_filename
@@ -33,14 +35,15 @@ from bzrlib.msgeditor import (
     edit_commit_message_encoded
 )
 from bzrlib.tests import (
-    iter_suite_tests,
-    probe_bad_non_ascii,
-    split_suite_by_re,
+    TestCaseInTempDir,
     TestCaseWithTransport,
     TestNotApplicable,
     TestSkipped,
+    multiply_tests,
+    probe_bad_non_ascii,
+    split_suite_by_re,
     )
-from bzrlib.tests.EncodingAdapter import EncodingTestAdapter
+from bzrlib.tests.EncodingAdapter import encoding_scenarios
 from bzrlib.trace import mutter
 
 
@@ -48,9 +51,7 @@ def load_tests(standard_tests, module, loader):
     """Parameterize the test for tempfile creation with different encodings."""
     to_adapt, result = split_suite_by_re(standard_tests,
         "test__create_temp_file_with_commit_template_in_unicode_dir")
-    for test in iter_suite_tests(to_adapt):
-        result.addTests(EncodingTestAdapter().adapt(test))
-    return result
+    return multiply_tests(to_adapt, encoding_scenarios, result)
 
 
 class MsgEditorTest(TestCaseWithTransport):
@@ -67,7 +68,7 @@ class MsgEditorTest(TestCaseWithTransport):
                 "filesystem encoding %s" % sys.getfilesystemencoding())
         working_tree.add(filename)
         return working_tree
-    
+
     def test_commit_template(self):
         """Test building a commit message template"""
         working_tree = self.make_uncommitted_tree()
@@ -79,6 +80,38 @@ added:
   hell\u00d8
 """)
 
+    def make_multiple_pending_tree(self):
+        from bzrlib import config
+        config.GlobalConfig().set_user_option('email',
+                                              'Bilbo Baggins <bb@hobbit.net>')
+        tree = self.make_branch_and_tree('a')
+        tree.commit('Initial checkin.', timestamp=1230912900, timezone=0)
+        tree2 = tree.bzrdir.clone('b').open_workingtree()
+        tree.commit('Minor tweak.', timestamp=1231977840, timezone=0)
+        tree2.commit('Feature X work.', timestamp=1233186240, timezone=0)
+        tree3 = tree2.bzrdir.clone('c').open_workingtree()
+        tree2.commit('Feature X finished.', timestamp=1233187680, timezone=0)
+        tree3.commit('Feature Y, based on initial X work.',
+                     timestamp=1233285960, timezone=0)
+        tree.merge_from_branch(tree2.branch)
+        tree.merge_from_branch(tree3.branch, force=True)
+        return tree
+
+    def test_commit_template_pending_merges(self):
+        """Test building a commit message template when there are pending
+        merges.  The commit message should show all pending merge revisions,
+        as does 'status -v', not only the merge tips.
+        """
+        working_tree = self.make_multiple_pending_tree()
+        template = msgeditor.make_commit_message_template(working_tree, None)
+        self.assertEqualDiff(template,
+u"""\
+pending merges:
+  Bilbo Baggins 2009-01-29 Feature X finished.
+    Bilbo Baggins 2009-01-28 Feature X work.
+  Bilbo Baggins 2009-01-30 Feature Y, based on initial X work.
+""")
+
     def test_commit_template_encoded(self):
         """Test building a commit message template"""
         working_tree = self.make_uncommitted_tree()
@@ -109,19 +142,21 @@ added:
   hell\u00d8
 """.encode('utf8') in template)
 
-    def test_run_editor(self):
+    def make_do_nothing_editor(self):
         if sys.platform == "win32":
             f = file('fed.bat', 'w')
             f.write('@rem dummy fed')
             f.close()
-            os.environ['BZR_EDITOR'] = 'fed.bat'
+            return 'fed.bat'
         else:
             f = file('fed.sh', 'wb')
             f.write('#!/bin/sh\n')
             f.close()
             os.chmod('fed.sh', 0755)
-            os.environ['BZR_EDITOR'] = './fed.sh'
+            return './fed.sh'
 
+    def test_run_editor(self):
+        os.environ['BZR_EDITOR'] = self.make_do_nothing_editor()
         self.assertEqual(True, msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
@@ -216,6 +251,7 @@ if len(sys.argv) == 2:
             f.close()
 
             editors = list(msgeditor._get_editor())
+            editors = [editor for (editor, cfg_src) in editors]
 
             self.assertEqual(['bzr_editor', 'config_editor', 'visual',
                               'editor'], editors[:4])
@@ -241,6 +277,32 @@ if len(sys.argv) == 2:
             else:
                 os.environ['EDITOR'] = editor
 
+    def test__run_editor_EACCES(self):
+        """If running a configured editor raises EACESS, the user is warned."""
+        os.environ['BZR_EDITOR'] = 'eacces.py'
+        f = file('eacces.py', 'wb')
+        f.write('# Not a real editor')
+        f.close()
+        # Make the fake editor unreadable (and unexecutable)
+        os.chmod('eacces.py', 0)
+        # Set $EDITOR so that _run_editor will terminate before trying real
+        # editors.
+        os.environ['EDITOR'] = self.make_do_nothing_editor()
+        # Call _run_editor, capturing mutter.warning calls.
+        warnings = []
+        def warning(*args):
+            if len(args) > 1:
+                warnings.append(args[0] % args[1:])
+            else:
+                warnings.append(args[0])
+        _warning = trace.warning
+        trace.warning = warning
+        try:
+            msgeditor._run_editor('')
+        finally:
+            trace.warning = _warning
+        self.assertStartsWith(warnings[0], 'Could not start editor "eacces.py"')
+
     def test__create_temp_file_with_commit_template(self):
         # check that commit template written properly
         # and has platform native line-endings (CRLF on win32)
@@ -257,8 +319,7 @@ if len(sys.argv) == 2:
         self.assertFileEqual(expected, msgfilename)
 
     def test__create_temp_file_with_commit_template_in_unicode_dir(self):
-        from bzrlib.tests.test_diff import UnicodeFilename
-        self.requireFeature(UnicodeFilename)
+        self.requireFeature(tests.UnicodeFilenameFeature)
         if hasattr(self, 'info'):
             os.mkdir(self.info['directory'])
             os.chdir(self.info['directory'])
@@ -280,10 +341,10 @@ if len(sys.argv) == 2:
             # LANG env variable has no effect on Windows
             # but some characters anyway cannot be represented
             # in default user encoding
-            char = probe_bad_non_ascii(bzrlib.user_encoding)
+            char = probe_bad_non_ascii(osutils.get_user_encoding())
             if char is None:
                 raise TestSkipped('Cannot find suitable non-ascii character '
-                    'for user_encoding (%s)' % bzrlib.user_encoding)
+                    'for user_encoding (%s)' % osutils.get_user_encoding())
 
             self.make_fake_editor(message=char)
 
@@ -292,3 +353,36 @@ if len(sys.argv) == 2:
                               msgeditor.edit_commit_message, '')
         finally:
             osutils.set_or_unset_env('LANG', old_env)
+
+    def test_generate_commit_message_template_no_hooks(self):
+        commit_obj = commit.Commit()
+        self.assertIs(None,
+            msgeditor.generate_commit_message_template(commit_obj))
+
+    def test_generate_commit_message_template_hook(self):
+        msgeditor.hooks.install_named_hook("commit_message_template",
+                lambda commit_obj, msg: "save me some typing\n", None)
+        commit_obj = commit.Commit()
+        self.assertEquals("save me some typing\n",
+            msgeditor.generate_commit_message_template(commit_obj))
+
+
+# GZ 2009-11-17: This wants moving to osutils when the errno checking code is
+class TestPlatformErrnoWorkarounds(TestCaseInTempDir):
+    """Ensuring workarounds enshrined in code actually serve a purpose"""
+
+    def test_subprocess_call_bad_file(self):
+        if sys.platform != "win32":
+            raise TestNotApplicable("Workarounds for windows only")
+        import subprocess, errno
+        ERROR_BAD_EXE_FORMAT = 193
+        file("textfile.txt", "w").close()
+        e = self.assertRaises(WindowsError, subprocess.call, "textfile.txt")
+        # Python2.4 used the 'winerror' as the errno, which confuses a lot of
+        # our error trapping code. Make sure that we understand the mapping
+        # correctly.
+        if sys.version_info >= (2, 5):
+            self.assertEqual(e.errno, errno.ENOEXEC)
+            self.assertEqual(e.winerror, ERROR_BAD_EXE_FORMAT)
+        else:
+            self.assertEqual(e.errno, ERROR_BAD_EXE_FORMAT)
