@@ -1,6 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
-#   and others
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,26 +12,34 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Symbol versioning tests."""
 
-import bzrlib.symbol_versioning as symbol_versioning
+import warnings
+
+from bzrlib import symbol_versioning
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_in,
+    deprecated_list,
+    deprecated_method,
+    )
 from bzrlib.tests import TestCase
 
 
-@symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
-def deprecated_function():
+@deprecated_function(deprecated_in((0, 7, 0)))
+def sample_deprecated_function():
     """Deprecated function docstring."""
     return 1
 
 
-a_deprecated_list = symbol_versioning.deprecated_list(symbol_versioning.zero_nine,
+a_deprecated_list = symbol_versioning.deprecated_list(deprecated_in((0, 9, 0)),
     'a_deprecated_list', ['one'], extra="Don't use me")
 
 
 a_deprecated_dict = symbol_versioning.DeprecatedDict(
-    symbol_versioning.zero_fourteen,
+    deprecated_in((0, 14, 0)),
     'a_deprecated_dict',
     dict(a=42),
     advice='Pull the other one!',
@@ -49,16 +55,16 @@ class TestDeprecationWarnings(TestCase):
         super(TestDeprecationWarnings, self).setUp()
         self._warnings = []
     
-    @symbol_versioning.deprecated_method(symbol_versioning.zero_seven)
+    @deprecated_method(deprecated_in((0, 7, 0)))
     def deprecated_method(self):
         """Deprecated method docstring.
-        
+
         This might explain stuff.
         """
         return 1
 
     @staticmethod
-    @symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
+    @deprecated_function(deprecated_in((0, 7, 0)))
     def deprecated_static():
         """Deprecated static."""
         return 1
@@ -70,11 +76,11 @@ class TestDeprecationWarnings(TestCase):
         expected_warning = (
             "bzrlib.tests.test_symbol_versioning."
             "deprecated_static "
-            "was deprecated in version 0.7.", DeprecationWarning, 2)
+            "was deprecated in version 0.7.0.", DeprecationWarning, 2)
         expected_docstring = (
             'Deprecated static.\n'
             '\n'
-            'This function was deprecated in version 0.7.\n'
+            'This function was deprecated in version 0.7.0.\n'
             )
         self.check_deprecated_callable(
             expected_warning, expected_docstring,
@@ -86,13 +92,14 @@ class TestDeprecationWarnings(TestCase):
         expected_warning = (
             "bzrlib.tests.test_symbol_versioning."
             "TestDeprecationWarnings.deprecated_method "
-            "was deprecated in version 0.7.", DeprecationWarning, 2)
-        expected_docstring = ('Deprecated method docstring.\n'
-                              '        \n'
-                              '        This might explain stuff.\n'
-                              '        \n'
-                              '        This method was deprecated in version 0.7.\n'
-                              '        ')
+            "was deprecated in version 0.7.0.", DeprecationWarning, 2)
+        expected_docstring = (
+            'Deprecated method docstring.\n'
+            '\n'
+            '        This might explain stuff.\n'
+            '        \n'
+            '        This method was deprecated in version 0.7.0.\n'
+            '        ')
         self.check_deprecated_callable(expected_warning, expected_docstring,
                                        "deprecated_method",
                                        "bzrlib.tests.test_symbol_versioning",
@@ -100,20 +107,20 @@ class TestDeprecationWarnings(TestCase):
 
     def test_deprecated_function(self):
         expected_warning = (
-            "bzrlib.tests.test_symbol_versioning.deprecated_function "
-            "was deprecated in version 0.7.", DeprecationWarning, 2)
+            "bzrlib.tests.test_symbol_versioning.sample_deprecated_function "
+            "was deprecated in version 0.7.0.", DeprecationWarning, 2)
         expected_docstring = ('Deprecated function docstring.\n'
                               '\n'
-                              'This function was deprecated in version 0.7.\n'
+                              'This function was deprecated in version 0.7.0.\n'
                               )
         self.check_deprecated_callable(expected_warning, expected_docstring,
-                                       "deprecated_function",
+                                       "sample_deprecated_function",
                                        "bzrlib.tests.test_symbol_versioning",
-                                       deprecated_function)
+                                       sample_deprecated_function)
 
     def test_deprecated_list(self):
         expected_warning = (
-            "Modifying a_deprecated_list was deprecated in version 0.9."
+            "Modifying a_deprecated_list was deprecated in version 0.9.0."
             " Don't use me", DeprecationWarning, 3)
         old_warning_method = symbol_versioning.warn
         try:
@@ -151,7 +158,7 @@ class TestDeprecationWarnings(TestCase):
 
     def test_deprecated_dict(self):
         expected_warning = (
-            "access to a_deprecated_dict was deprecated in version 0.14."
+            "access to a_deprecated_dict was deprecated in version 0.14.0."
             " Pull the other one!", DeprecationWarning, 2)
         old_warning_method = symbol_versioning.warn
         try:
@@ -172,6 +179,9 @@ class TestDeprecationWarnings(TestCase):
     def check_deprecated_callable(self, expected_warning, expected_docstring,
                                   expected_name, expected_module,
                                   deprecated_callable):
+        if __doc__ is None:
+            # With -OO the docstring should just be the deprecated version
+            expected_docstring = expected_docstring.split('\n')[-2].lstrip()
         old_warning_method = symbol_versioning.warn
         try:
             symbol_versioning.set_warning_method(self.capture_warning)
@@ -186,7 +196,7 @@ class TestDeprecationWarnings(TestCase):
             self.assertTrue(deprecated_callable.is_deprecated)
         finally:
             symbol_versioning.set_warning_method(old_warning_method)
-    
+
     def test_deprecated_passed(self):
         self.assertEqual(True, symbol_versioning.deprecated_passed(None))
         self.assertEqual(True, symbol_versioning.deprecated_passed(True))
@@ -199,11 +209,90 @@ class TestDeprecationWarnings(TestCase):
         """We can get a deprecation string for a method or function."""
         self.assertEqual('bzrlib.tests.test_symbol_versioning.'
             'TestDeprecationWarnings.test_deprecation_string was deprecated in '
-            'version 0.11.',
+            'version 0.11.0.',
             symbol_versioning.deprecation_string(
-            self.test_deprecation_string, symbol_versioning.zero_eleven))
+            self.test_deprecation_string,
+            deprecated_in((0, 11, 0))))
         self.assertEqual('bzrlib.symbol_versioning.deprecated_function was '
-            'deprecated in version 0.11.',
+            'deprecated in version 0.11.0.',
             symbol_versioning.deprecation_string(
                 symbol_versioning.deprecated_function,
-                symbol_versioning.zero_eleven))
+                deprecated_in((0, 11, 0))))
+
+
+class TestSuppressAndActivate(TestCase):
+
+    def setUp(self):
+        TestCase.setUp(self)
+        existing_filters = list(warnings.filters)
+        def restore():
+            warnings.filters[:] = existing_filters
+        self.addCleanup(restore)
+        # Clean out the filters so we have a clean slate.
+        warnings.resetwarnings()
+
+    def assertFirstWarning(self, action, category):
+        """Test the first warning in the filters is correct"""
+        first = warnings.filters[0]
+        self.assertEqual((action, category), (first[0], first[2]))
+
+    def test_suppress_deprecation_warnings(self):
+        """suppress_deprecation_warnings sets DeprecationWarning to ignored."""
+        symbol_versioning.suppress_deprecation_warnings()
+        self.assertFirstWarning('ignore', DeprecationWarning)
+
+    def test_set_restore_filters(self):
+        original_filters = warnings.filters[:]
+        symbol_versioning.suppress_deprecation_warnings()()
+        self.assertEqual(original_filters, warnings.filters)
+
+    def test_suppress_deprecation_with_warning_filter(self):
+        """don't suppress if we already have a filter"""
+        warnings.filterwarnings('error', category=Warning)
+        self.assertFirstWarning('error', Warning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.suppress_deprecation_warnings(override=False)
+        self.assertFirstWarning('error', Warning)
+        self.assertEqual(1, len(warnings.filters))
+
+    def test_suppress_deprecation_with_filter(self):
+        """don't suppress if we already have a filter"""
+        warnings.filterwarnings('error', category=DeprecationWarning)
+        self.assertFirstWarning('error', DeprecationWarning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.suppress_deprecation_warnings(override=False)
+        self.assertFirstWarning('error', DeprecationWarning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.suppress_deprecation_warnings(override=True)
+        self.assertFirstWarning('ignore', DeprecationWarning)
+        self.assertEqual(2, len(warnings.filters))
+
+    def test_activate_deprecation_no_error(self):
+        # First nuke the filters, so we know it is clean
+        symbol_versioning.activate_deprecation_warnings()
+        self.assertFirstWarning('default', DeprecationWarning)
+
+    def test_activate_deprecation_with_error(self):
+        # First nuke the filters, so we know it is clean
+        # Add a warning == error rule
+        warnings.filterwarnings('error', category=Warning)
+        self.assertFirstWarning('error', Warning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.activate_deprecation_warnings(override=False)
+        # There should not be a new warning
+        self.assertFirstWarning('error', Warning)
+        self.assertEqual(1, len(warnings.filters))
+
+    def test_activate_deprecation_with_DW_error(self):
+        # First nuke the filters, so we know it is clean
+        # Add a warning == error rule
+        warnings.filterwarnings('error', category=DeprecationWarning)
+        self.assertFirstWarning('error', DeprecationWarning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.activate_deprecation_warnings(override=False)
+        # There should not be a new warning
+        self.assertFirstWarning('error', DeprecationWarning)
+        self.assertEqual(1, len(warnings.filters))
+        symbol_versioning.activate_deprecation_warnings(override=True)
+        self.assertFirstWarning('default', DeprecationWarning)
+        self.assertEqual(2, len(warnings.filters))
