@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
 import urllib
@@ -43,12 +43,12 @@ class HttpTransport_urllib(http.HttpTransportBase):
 
     def __init__(self, base, _from_transport=None):
         super(HttpTransport_urllib, self).__init__(
-            base, _from_transport=_from_transport)
+            base, 'urllib', _from_transport=_from_transport)
         if _from_transport is not None:
             self._opener = _from_transport._opener
         else:
-            self._opener = self._opener_class()
-        self._remote_is_at_least_1_2 = True
+            self._opener = self._opener_class(
+                report_activity=self._report_activity)
 
     def _perform(self, request):
         """Send the request to the server and handles common errors.
@@ -87,12 +87,11 @@ class HttpTransport_urllib(http.HttpTransportBase):
             self._update_credentials((request.auth, request.proxy_auth))
 
         code = response.code
-        if request.follow_redirections is False \
-                and code in (301, 302, 303, 307):
+        if (request.follow_redirections is False
+            and code in (301, 302, 303, 307)):
             raise errors.RedirectRequested(request.get_full_url(),
                                            request.redirected_to,
-                                           is_permanent=(code == 301),
-                                           qual_proto=self._scheme)
+                                           is_permanent=(code == 301))
 
         if request.redirected_to is not None:
             trace.mutter('redirected from: %s to: %s' % (request.get_full_url(),
@@ -102,7 +101,6 @@ class HttpTransport_urllib(http.HttpTransportBase):
 
     def _get(self, relpath, offsets, tail_amount=0):
         """See HttpTransport._get"""
-
         abspath = self._remote_path(relpath)
         headers = {}
         accepted_errors = [200, 404]
@@ -133,7 +131,12 @@ class HttpTransport_urllib(http.HttpTransportBase):
 
     def _post(self, body_bytes):
         abspath = self._remote_path('.bzr/smart')
-        response = self._perform(Request('POST', abspath, body_bytes))
+        # We include 403 in accepted_errors so that send_http_smart_request can
+        # handle a 403.  Otherwise a 403 causes an unhandled TransportError.
+        response = self._perform(
+            Request('POST', abspath, body_bytes,
+                    {'Content-Type': 'application/octet-stream'},
+                    accepted_errors=[200, 403]))
         code = response.code
         data = handle_response(abspath, code, response.info(), response)
         return code, data
@@ -164,6 +167,11 @@ class HttpTransport_urllib(http.HttpTransportBase):
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    from bzrlib.tests.http_server import HttpServer_urllib
-    return [(HttpTransport_urllib, HttpServer_urllib),
-            ]
+    from bzrlib import tests
+    from bzrlib.tests import http_server
+    permutations = [(HttpTransport_urllib, http_server.HttpServer_urllib),]
+    if tests.HTTPSServerFeature.available():
+        from bzrlib.tests import https_server
+        permutations.append((HttpTransport_urllib,
+                             https_server.HTTPSServer_urllib))
+    return permutations
