@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2006-2010 Robey Pointer <robey@lag.net>
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -94,7 +94,10 @@ class SSHVendorManager(object):
             try:
                 vendor = self._ssh_vendors[vendor_name]
             except KeyError:
-                raise errors.UnknownSSH(vendor_name)
+                vendor = self._get_vendor_from_path(vendor_name)
+                if vendor is None:
+                    raise errors.UnknownSSH(vendor_name)
+                vendor.executable_path = vendor_name
             return vendor
         return None
 
@@ -110,7 +113,7 @@ class SSHVendorManager(object):
             stdout = stderr = ''
         return stdout + stderr
 
-    def _get_vendor_by_version_string(self, version, args):
+    def _get_vendor_by_version_string(self, version, progname):
         """Return the vendor or None based on output from the subprocess.
 
         :param version: The output of 'ssh -V' like command.
@@ -123,25 +126,27 @@ class SSHVendorManager(object):
         elif 'SSH Secure Shell' in version:
             trace.mutter('ssh implementation is SSH Corp.')
             vendor = SSHCorpSubprocessVendor()
-        # Auto-detect of plink vendor disabled, on Windows recommended
-        # default ssh-client is paramiko
-        # see https://bugs.launchpad.net/bugs/414743
-        #~elif 'plink' in version and args[0] == 'plink':
-        #~    # Checking if "plink" was the executed argument as Windows
-        #~    # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
-        #~    # version.  See https://bugs.launchpad.net/bzr/+bug/107155
-        #~    trace.mutter("ssh implementation is Putty's plink.")
-        #~    vendor = PLinkSubprocessVendor()
+        # As plink user prompts are not handled currently, don't auto-detect
+        # it by inspection below, but keep this vendor detection for if a path
+        # is given in BZR_SSH. See https://bugs.launchpad.net/bugs/414743
+        elif 'plink' in version and progname == 'plink':
+            # Checking if "plink" was the executed argument as Windows
+            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+            # version.  See https://bugs.launchpad.net/bzr/+bug/107155
+            trace.mutter("ssh implementation is Putty's plink.")
+            vendor = PLinkSubprocessVendor()
         return vendor
 
     def _get_vendor_by_inspection(self):
         """Return the vendor or None by checking for known SSH implementations."""
-        for args in (['ssh', '-V'], ['plink', '-V']):
-            version = self._get_ssh_version_string(args)
-            vendor = self._get_vendor_by_version_string(version, args)
-            if vendor is not None:
-                return vendor
-        return None
+        version = self._get_ssh_version_string(['ssh', '-V'])
+        return self._get_vendor_by_version_string(version, "ssh")
+
+    def _get_vendor_from_path(self, path):
+        """Return the vendor or None using the program at the given path"""
+        version = self._get_ssh_version_string([path, '-V'])
+        return self._get_vendor_by_version_string(version, 
+            os.path.splitext(os.path.basename(path))[0])
 
     def get_vendor(self, environment=None):
         """Find out what version of SSH is on the system.
@@ -168,12 +173,15 @@ register_default_ssh_vendor = _ssh_vendor_manager.register_default_vendor
 register_ssh_vendor = _ssh_vendor_manager.register_vendor
 
 
-def _ignore_sigint():
+def _ignore_signals():
     # TODO: This should possibly ignore SIGHUP as well, but bzr currently
     # doesn't handle it itself.
     # <https://launchpad.net/products/bzr/+bug/41433/+index>
     import signal
     signal.signal(signal.SIGINT, signal.SIG_IGN)
+    # GZ 2010-02-19: Perhaps make this check if breakin is installed instead
+    if signal.getsignal(signal.SIGQUIT) != signal.SIG_DFL:
+        signal.signal(signal.SIGQUIT, signal.SIG_IGN)
 
 
 class SocketAsChannelAdapter(object):
@@ -231,8 +239,7 @@ class SSHVendor(object):
     def connect_ssh(self, username, password, host, port, command):
         """Make an SSH connection.
 
-        :returns: something with a `close` method, and a `get_filelike_channels`
-            method that returns a pair of (read, write) filelike objects.
+        :returns: an SSHConnection.
         """
         raise NotImplementedError(self.connect_ssh)
 
@@ -261,17 +268,6 @@ class LoopbackVendor(SSHVendor):
 register_ssh_vendor('loopback', LoopbackVendor())
 
 
-class _ParamikoSSHConnection(object):
-    def __init__(self, channel):
-        self.channel = channel
-
-    def get_filelike_channels(self):
-        return self.channel.makefile('rb'), self.channel.makefile('wb')
-
-    def close(self):
-        return self.channel.close()
-
-
 class ParamikoVendor(SSHVendor):
     """Vendor that uses paramiko."""
 
@@ -355,11 +351,24 @@ class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
 
     def _connect(self, argv):
-        proc = subprocess.Popen(argv,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
+        # Attempt to make a socketpair to use as stdin/stdout for the SSH
+        # subprocess.  We prefer sockets to pipes because they support
+        # non-blocking short reads, allowing us to optimistically read 64k (or
+        # whatever) chunks.
+        try:
+            my_sock, subproc_sock = socket.socketpair()
+        except (AttributeError, socket.error):
+            # This platform doesn't support socketpair(), so just use ordinary
+            # pipes instead.
+            stdin = stdout = subprocess.PIPE
+            my_sock, subproc_sock = None, None
+        else:
+            stdin = stdout = subproc_sock
+        proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
                                 **os_specific_subprocess_params())
-        return SSHSubprocess(proc)
+        if subproc_sock is not None:
+            subproc_sock.close()
+        return SSHSubprocessConnection(proc, sock=my_sock)
 
     def connect_sftp(self, username, password, host, port):
         try:
@@ -404,9 +413,11 @@ class SubprocessVendor(SSHVendor):
 class OpenSSHSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'ssh' executable from OpenSSH."""
 
+    executable_path = 'ssh'
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        args = ['ssh',
+        args = [self.executable_path,
                 '-oForwardX11=no', '-oForwardAgent=no',
                 '-oClearAllForwardings=yes', '-oProtocol=2',
                 '-oNoHostAuthenticationForLocalhost=yes']
@@ -426,9 +437,11 @@ register_ssh_vendor('openssh', OpenSSHSubprocessVendor())
 class SSHCorpSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'ssh' executable from SSH Corporation."""
 
+    executable_path = 'ssh'
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        args = ['ssh', '-x']
+        args = [self.executable_path, '-x']
         if port is not None:
             args.extend(['-p', str(port)])
         if username is not None:
@@ -439,15 +452,17 @@ class SSHCorpSubprocessVendor(SubprocessVendor):
             args.extend([host] + command)
         return args
 
-register_ssh_vendor('ssh', SSHCorpSubprocessVendor())
+register_ssh_vendor('sshcorp', SSHCorpSubprocessVendor())
 
 
 class PLinkSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'plink' executable from Putty."""
 
+    executable_path = 'plink'
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        args = ['plink', '-x', '-a', '-ssh', '-2', '-batch']
+        args = [self.executable_path, '-x', '-a', '-ssh', '-2', '-batch']
         if port is not None:
             args.extend(['-P', str(port)])
         if username is not None:
@@ -623,28 +638,110 @@ def os_specific_subprocess_params():
         # Running it in a separate process group is not good because then it
         # can't get non-echoed input of a password or passphrase.
         # <https://launchpad.net/products/bzr/+bug/40508>
-        return {'preexec_fn': _ignore_sigint,
+        return {'preexec_fn': _ignore_signals,
                 'close_fds': True,
                 }
 
+import weakref
+_subproc_weakrefs = set()
 
-class SSHSubprocess(object):
-    """A socket-like object that talks to an ssh subprocess via pipes."""
+def _close_ssh_proc(proc, sock):
+    """Carefully close stdin/stdout and reap the SSH process.
 
-    def __init__(self, proc):
-        self.proc = proc
+    If the pipes are already closed and/or the process has already been
+    wait()ed on, that's ok, and no error is raised.  The goal is to do our best
+    to clean up (whether or not a clean up was already tried).
+    """
+    funcs = []
+    for closeable in (proc.stdin, proc.stdout, sock):
+        # We expect that either proc (a subprocess.Popen) will have stdin and
+        # stdout streams to close, or that we will have been passed a socket to
+        # close, with the option not in use being None.
+        if closeable is not None:
+            funcs.append(closeable.close)
+    funcs.append(proc.wait)
+    for func in funcs:
+        try:
+            func()
+        except OSError:
+            # It's ok for the pipe to already be closed, or the process to
+            # already be finished.
+            continue
 
-    def send(self, data):
-        return os.write(self.proc.stdin.fileno(), data)
 
-    def recv(self, count):
-        return os.read(self.proc.stdout.fileno(), count)
+class SSHConnection(object):
+    """Abstract base class for SSH connections."""
+
+    def get_sock_or_pipes(self):
+        """Returns a (kind, io_object) pair.
+
+        If kind == 'socket', then io_object is a socket.
+
+        If kind == 'pipes', then io_object is a pair of file-like objects
+        (read_from, write_to).
+        """
+        raise NotImplementedError(self.get_sock_or_pipes)
 
     def close(self):
-        self.proc.stdin.close()
-        self.proc.stdout.close()
-        self.proc.wait()
+        raise NotImplementedError(self.close)
+
+
+class SSHSubprocessConnection(SSHConnection):
+    """A connection to an ssh subprocess via pipes or a socket.
+
+    This class is also socket-like enough to be used with
+    SocketAsChannelAdapter (it has 'send' and 'recv' methods).
+    """
+
+    def __init__(self, proc, sock=None):
+        """Constructor.
+
+        :param proc: a subprocess.Popen
+        :param sock: if proc.stdin/out is a socket from a socketpair, then sock
+            should bzrlib's half of that socketpair.  If not passed, proc's
+            stdin/out is assumed to be ordinary pipes.
+        """
+        self.proc = proc
+        self._sock = sock
+        # Add a weakref to proc that will attempt to do the same as self.close
+        # to avoid leaving processes lingering indefinitely.
+        def terminate(ref):
+            _subproc_weakrefs.remove(ref)
+            _close_ssh_proc(proc, sock)
+        _subproc_weakrefs.add(weakref.ref(self, terminate))
+
+    def send(self, data):
+        if self._sock is not None:
+            return self._sock.send(data)
+        else:
+            return os.write(self.proc.stdin.fileno(), data)
+
+    def recv(self, count):
+        if self._sock is not None:
+            return self._sock.recv(count)
+        else:
+            return os.read(self.proc.stdout.fileno(), count)
+
+    def close(self):
+        _close_ssh_proc(self.proc, self._sock)
+
+    def get_sock_or_pipes(self):
+        if self._sock is not None:
+            return 'socket', self._sock
+        else:
+            return 'pipes', (self.proc.stdout, self.proc.stdin)
+
+
+class _ParamikoSSHConnection(SSHConnection):
+    """An SSH connection via paramiko."""
+
+    def __init__(self, channel):
+        self.channel = channel
+
+    def get_sock_or_pipes(self):
+        return ('socket', self.channel)
+
+    def close(self):
+        return self.channel.close()
 
-    def get_filelike_channels(self):
-        return (self.proc.stdout, self.proc.stdin)
 
