@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ from bzrlib import (
     lockdir,
     osutils,
     revision as _mod_revision,
+    trace,
     urlutils,
     versionedfile,
     weave,
@@ -53,7 +54,6 @@ from bzrlib.repository import (
     RepositoryFormat,
     )
 from bzrlib.store.text import TextStore
-from bzrlib.trace import mutter
 from bzrlib.tuned_gzip import GzipFile, bytes_to_gzip
 from bzrlib.versionedfile import (
     AbsentContentFactory,
@@ -106,7 +106,8 @@ class AllInOneRepository(Repository):
     def _all_possible_ids(self):
         """Return all the possible revisions that we could find."""
         if 'evil' in debug.debug_flags:
-            mutter_callsite(3, "_all_possible_ids scales with size of history.")
+            trace.mutter_callsite(
+                3, "_all_possible_ids scales with size of history.")
         return [key[-1] for key in self.inventories.keys()]
 
     @needs_read_lock
@@ -176,7 +177,7 @@ class AllInOneRepository(Repository):
         :param new_value: True to restore the default, False to disable making
                           working trees.
         """
-        raise errors.RepositoryUpgradeRequired(self.bzrdir.root_transport.base)
+        raise errors.RepositoryUpgradeRequired(self.user_url)
 
     def make_working_trees(self):
         """Returns the policy for making working trees on new branches."""
@@ -199,7 +200,8 @@ class WeaveMetaDirRepository(MetaDirVersionedFileRepository):
     def _all_possible_ids(self):
         """Return all the possible revisions that we could find."""
         if 'evil' in debug.debug_flags:
-            mutter_callsite(3, "_all_possible_ids scales with size of history.")
+            trace.mutter_callsite(
+                3, "_all_possible_ids scales with size of history.")
         return [key[-1] for key in self.inventories.keys()]
 
     @needs_read_lock
@@ -286,7 +288,7 @@ class PreSplitOutRepositoryFormat(RepositoryFormat):
         weavefile.write_weave_v5(weave.Weave(), sio)
         empty_weave = sio.getvalue()
 
-        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        trace.mutter('creating repository in %s.', a_bzrdir.transport.base)
 
         # FIXME: RBC 20060125 don't peek under the covers
         # NB: no need to escape relative paths that are url safe.
@@ -302,7 +304,9 @@ class PreSplitOutRepositoryFormat(RepositoryFormat):
                 mode=a_bzrdir._get_file_mode())
         finally:
             control_files.unlock()
-        return self.open(a_bzrdir, _found=True)
+        repository = self.open(a_bzrdir, _found=True)
+        self._run_post_repo_init_hooks(repository, a_bzrdir, shared)
+        return repository
 
     def open(self, a_bzrdir, _found=False):
         """See RepositoryFormat.open()."""
@@ -524,7 +528,7 @@ class RepositoryFormat7(MetaDirRepositoryFormat):
         weavefile.write_weave_v5(weave.Weave(), sio)
         empty_weave = sio.getvalue()
 
-        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        trace.mutter('creating repository in %s.', a_bzrdir.transport.base)
         dirs = ['revision-store', 'weaves']
         files = [('inventory.weave', StringIO(empty_weave)),
                  ]
@@ -688,7 +692,7 @@ class RevisionTextStore(TextVersionedFiles):
             path, ext = os.path.splitext(relpath)
             if ext == '.gz':
                 relpath = path
-            if '.sig' not in relpath:
+            if not relpath.endswith('.sig'):
                 relpaths.add(relpath)
         paths = list(relpaths)
         return set([self._mapper.unmap(path) for path in paths])
