@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,17 +22,18 @@ from cStringIO import StringIO
 
 from bzrlib import (
     bzrdir,
-    errors
+    errors,
+    tests,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import (BzrDir, BzrDirFormat, BzrDirMetaFormat1)
 from bzrlib.osutils import getcwd
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import script
 import bzrlib.urlutils as urlutils
 from bzrlib.workingtree import WorkingTree
 
 
-class TestLegacyFormats(TestCaseWithTransport):
+class TestLegacyFormats(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestLegacyFormats, self).setUp()
@@ -61,17 +62,16 @@ class TestLegacyFormats(TestCaseWithTransport):
                          'upgrade your branch at %s/.\n' % cwd, err)
 
 
-class TestBoundBranches(TestCaseWithTransport):
+class TestBoundBranches(tests.TestCaseWithTransport):
 
     def create_branches(self):
-        self.build_tree(['base/', 'base/a', 'base/b'])
-
-        branch = self.init_meta_branch('base')
-        base_tree = branch.bzrdir.open_workingtree()
+        base_tree = self.make_branch_and_tree('base')
         base_tree.lock_write()
+        self.build_tree(['base/a', 'base/b'])
         base_tree.add(['a', 'b'])
         base_tree.commit('init')
         base_tree.unlock()
+        branch = base_tree.branch
 
         child_tree = branch.create_checkout('child')
 
@@ -86,12 +86,11 @@ class TestBoundBranches(TestCaseWithTransport):
             val, len(BzrDir.open(loc).open_branch().revision_history()))
 
     def test_simple_binding(self):
-        self.build_tree(['base/', 'base/a', 'base/b'])
-
-        branch = self.init_meta_branch('base')
-        tree = branch.bzrdir.open_workingtree()
+        tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/a', 'base/b'])
         tree.add('a', 'b')
         tree.commit(message='init')
+        branch = tree.branch
 
         tree.bzrdir.sprout('child')
 
@@ -131,10 +130,6 @@ class TestBoundBranches(TestCaseWithTransport):
         error = self.run_bzr('bind', retcode=3)[1]
         self.assertContainsRe(error, 'old locations')
 
-    def init_meta_branch(self, path):
-        format = bzrdir.format_registry.make_bzrdir('default')
-        return BzrDir.create_branch_convenience(path, format=format)
-
     def test_bound_commit(self):
         child_tree = self.create_branches()[1]
 
@@ -426,3 +421,39 @@ class TestBoundBranches(TestCaseWithTransport):
         # both the local and master should have been updated.
         self.check_revno(4)
         self.check_revno(4, '../base')
+
+    def test_bind_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/a', 'base/b'])
+        tree.add('a', 'b')
+        tree.commit(message='init')
+        branch = tree.branch
+        tree.bzrdir.sprout('child')
+        self.run_bzr('bind --directory=child base')
+        d = BzrDir.open('child')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind -d child')
+        self.assertEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind --directory child', retcode=3)
+
+
+class TestBind(script.TestCaseWithTransportAndScript):
+
+    def test_bind_when_bound(self):
+        self.run_script("""
+$ bzr init trunk
+$ bzr init copy
+$ cd copy
+$ bzr bind ../trunk
+$ bzr bind
+2>bzr: ERROR: Branch is already bound
+""")
+
+    def test_bind_before_bound(self):
+        self.run_script("""
+$ bzr init trunk
+$ cd trunk
+$ bzr bind
+2>bzr: ERROR: No location supplied and no previous location known
+""")
