@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,22 +12,20 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for the urlutils wrapper."""
 
 import os
-import re
 import sys
 
-from bzrlib import osutils, urlutils
-import bzrlib
-from bzrlib.errors import InvalidURL, InvalidURLJoin
+from bzrlib import osutils, urlutils, win32utils
+from bzrlib.errors import InvalidURL, InvalidURLJoin, InvalidRebaseURLs
 from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 
 
 class TestUrlToPath(TestCase):
-    
+
     def test_basename(self):
         # bzrlib.urlutils.basename
         # Test bzrlib.urlutils.split()
@@ -82,9 +80,9 @@ class TestUrlToPath(TestCase):
 
         # Local paths are assumed to *not* be escaped at all
         try:
-            u'uni/\xb5'.encode(bzrlib.user_encoding)
+            u'uni/\xb5'.encode(osutils.get_user_encoding())
         except UnicodeError:
-            # locale cannot handle unicode 
+            # locale cannot handle unicode
             pass
         else:
             norm_file('uni/%C2%B5', u'uni/\xb5')
@@ -115,6 +113,12 @@ class TestUrlToPath(TestCase):
         eq('http://host/ab/%C2%B5/%C2%B5',
             normalize_url(u'http://host/ab/%C2%B5/\xb5'))
 
+        # Unescape characters that don't need to be escaped
+        eq('http://host/~bob%2525-._',
+                normalize_url('http://host/%7Ebob%2525%2D%2E%5F'))
+        eq('http://host/~bob%2525-._',
+                normalize_url(u'http://host/%7Ebob%2525%2D%2E%5F'))
+
         # Normalize verifies URLs when they are not unicode
         # (indicating they did not come from the user)
         self.assertRaises(InvalidURL, normalize_url, 'http://host/\xb5')
@@ -152,7 +156,7 @@ class TestUrlToPath(TestCase):
         # Weird stuff
         # Can't have slashes or colons in the scheme
         test_one('/path/to/://foo', None)
-        test_one('path:path://foo', None)
+        test_one('scheme:stuff://foo', ('scheme', 'stuff://foo'))
         # Must have more than one character for scheme
         test_one('C://foo', None)
         test_one('ab://foo', ('ab', 'foo'))
@@ -191,38 +195,144 @@ class TestUrlToPath(TestCase):
             dirname('path/to/foo/', exclude_trailing_slash=False))
         self.assertEqual('path/..', dirname('path/../foo'))
         self.assertEqual('../path', dirname('../path/foo'))
+    
+    def test_is_url(self):
+        self.assertTrue(urlutils.is_url('http://foo/bar'))
+        self.assertTrue(urlutils.is_url('bzr+ssh://foo/bar'))
+        self.assertTrue(urlutils.is_url('lp:foo/bar'))
+        self.assertTrue(urlutils.is_url('file:///foo/bar'))
+        self.assertFalse(urlutils.is_url(''))
+        self.assertFalse(urlutils.is_url('foo'))
+        self.assertFalse(urlutils.is_url('foo/bar'))
+        self.assertFalse(urlutils.is_url('/foo'))
+        self.assertFalse(urlutils.is_url('/foo/bar'))
+        self.assertFalse(urlutils.is_url('C:/'))
+        self.assertFalse(urlutils.is_url('C:/foo'))
+        self.assertFalse(urlutils.is_url('C:/foo/bar'))
 
     def test_join(self):
         def test(expected, *args):
             joined = urlutils.join(*args)
             self.assertEqual(expected, joined)
 
+        # Test relative path joining
+        test('foo', 'foo') # relative fragment with nothing is preserved.
+        test('foo/bar', 'foo', 'bar')
+        test('http://foo/bar', 'http://foo', 'bar')
+        test('http://foo/bar', 'http://foo', '.', 'bar')
+        test('http://foo/baz', 'http://foo', 'bar', '../baz')
+        test('http://foo/bar/baz', 'http://foo', 'bar/baz')
+        test('http://foo/baz', 'http://foo', 'bar/../baz')
+        test('http://foo/baz', 'http://foo/bar/', '../baz')
+        test('lp:foo/bar', 'lp:foo', 'bar')
+        test('lp:foo/bar/baz', 'lp:foo', 'bar/baz')
+
+        # Absolute paths
+        test('http://foo', 'http://foo') # abs url with nothing is preserved.
+        test('http://bar', 'http://foo', 'http://bar')
+        test('sftp://bzr/foo', 'http://foo', 'bar', 'sftp://bzr/foo')
+        test('file:///bar', 'foo', 'file:///bar')
+        test('http://bar/', 'http://foo', 'http://bar/')
+        test('http://bar/a', 'http://foo', 'http://bar/a')
+        test('http://bar/a/', 'http://foo', 'http://bar/a/')
+        test('lp:bar', 'http://foo', 'lp:bar')
+        test('lp:bar', 'lp:foo', 'lp:bar')
+        test('file:///stuff', 'lp:foo', 'file:///stuff')
+
+        # From a base path
+        test('file:///foo', 'file:///', 'foo')
+        test('file:///bar/foo', 'file:///bar/', 'foo')
+        test('http://host/foo', 'http://host/', 'foo')
+        test('http://host/', 'http://host', '')
+
+        # Invalid joinings
+        # Cannot go above root
+        # Implicitly at root:
+        self.assertRaises(InvalidURLJoin, urlutils.join,
+                'http://foo', '../baz')
+        self.assertRaises(InvalidURLJoin, urlutils.join,
+                'http://foo', '/..')
+        # Joining from a path explicitly under the root.
+        self.assertRaises(InvalidURLJoin, urlutils.join,
+                'http://foo/a', '../../b')
+
+    def test_joinpath(self):
+        def test(expected, *args):
+            joined = urlutils.joinpath(*args)
+            self.assertEqual(expected, joined)
+
         # Test a single element
         test('foo', 'foo')
 
         # Test relative path joining
         test('foo/bar', 'foo', 'bar')
-        test('http://foo/bar', 'http://foo', 'bar')
-        test('http://foo/bar', 'http://foo', '.', 'bar')
-        test('http://foo/baz', 'http://foo', 'bar', '../baz')
-        test('http://foo/bar/baz', 'http://foo', 'bar/baz')
-        test('http://foo/baz', 'http://foo', 'bar/../baz')
+        test('foo/bar', 'foo', '.', 'bar')
+        test('foo/baz', 'foo', 'bar', '../baz')
+        test('foo/bar/baz', 'foo', 'bar/baz')
+        test('foo/baz', 'foo', 'bar/../baz')
 
-        # Absolute paths
-        test('http://bar', 'http://foo', 'http://bar')
-        test('sftp://bzr/foo', 'http://foo', 'bar', 'sftp://bzr/foo')
-        test('file:///bar', 'foo', 'file:///bar')
+        # Test joining to an absolute path
+        test('/foo', '/foo')
+        test('/foo', '/foo', '.')
+        test('/foo/bar', '/foo', 'bar')
+        test('/', '/foo', '..')
+
+        # Test joining with an absolute path
+        test('/bar', 'foo', '/bar')
+
+        # Test joining to a path with a trailing slash
+        test('foo/bar', 'foo/', 'bar')
 
-        # From a base path
-        test('file:///foo', 'file:///', 'foo')
-        test('file:///bar/foo', 'file:///bar/', 'foo')
-        test('http://host/foo', 'http://host/', 'foo')
-        test('http://host/', 'http://host', '')
-        
         # Invalid joinings
         # Cannot go above root
-        self.assertRaises(InvalidURLJoin, urlutils.join,
-                'http://foo', '../baz')
+        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '../baz')
+        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '..')
+        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '/..')
+
+    def test_join_segment_parameters_raw(self):
+        join_segment_parameters_raw = urlutils.join_segment_parameters_raw
+        self.assertEquals("/somedir/path", 
+            join_segment_parameters_raw("/somedir/path"))
+        self.assertEquals("/somedir/path,rawdata", 
+            join_segment_parameters_raw("/somedir/path", "rawdata"))
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters_raw, "/somedir/path",
+                "rawdata1,rawdata2,rawdata3")
+        self.assertEquals("/somedir/path,bla,bar",
+            join_segment_parameters_raw("/somedir/path", "bla", "bar"))
+        self.assertEquals("/somedir,exist=some/path,bla,bar",
+            join_segment_parameters_raw("/somedir,exist=some/path",
+                "bla", "bar"))
+        self.assertRaises(TypeError, join_segment_parameters_raw, 
+            "/somepath", 42)
+
+    def test_join_segment_parameters(self):
+        join_segment_parameters = urlutils.join_segment_parameters
+        self.assertEquals("/somedir/path", 
+            join_segment_parameters("/somedir/path", {}))
+        self.assertEquals("/somedir/path,key1=val1", 
+            join_segment_parameters("/somedir/path", {"key1": "val1"}))
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters, "/somedir/path",
+            {"branch": "brr,brr,brr"})
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters, "/somedir/path", {"key1=val1": "val2"})
+        self.assertEquals("/somedir/path,key1=val1,key2=val2",
+            join_segment_parameters("/somedir/path", {
+                "key1": "val1", "key2": "val2"}))
+        self.assertEquals("/somedir/path,key1=val1,key2=val2",
+            join_segment_parameters("/somedir/path,key1=val1", {
+                "key2": "val2"}))
+        self.assertEquals("/somedir/path,key1=val2",
+            join_segment_parameters("/somedir/path,key1=val1", {
+                "key1": "val2"}))
+        self.assertEquals("/somedir,exist=some/path,key1=val1",
+            join_segment_parameters("/somedir,exist=some/path",
+                {"key1": "val1"}))
+        self.assertEquals("/,key1=val1,key2=val2",
+            join_segment_parameters("/,key1=val1", {"key2": "val2"}))
+        self.assertRaises(TypeError,
+            join_segment_parameters, "/,key1=val1", {"foo": 42})
 
     def test_function_type(self):
         if sys.platform == 'win32':
@@ -243,6 +353,7 @@ class TestUrlToPath(TestCase):
             raise TestSkipped("local encoding cannot handle unicode")
 
         self.assertEqual('file:///path/to/r%C3%A4ksm%C3%B6rg%C3%A5s', result)
+        self.assertFalse(isinstance(result, unicode))
 
     def test_posix_local_path_from_url(self):
         from_url = urlutils._posix_local_path_from_url
@@ -252,8 +363,13 @@ class TestUrlToPath(TestCase):
             from_url('file:///path/to/r%C3%A4ksm%C3%B6rg%C3%A5s'))
         self.assertEqual(u'/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
+        self.assertEqual(u'/path/to/r\xe4ksm\xf6rg\xe5s',
+            from_url('file://localhost/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
 
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
+        self.assertRaises(
+            InvalidURL, from_url,
+            'file://remotehost/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s')
 
     def test_win32_local_path_to_url(self):
         to_url = urlutils._win32_local_path_to_url
@@ -268,12 +384,15 @@ class TestUrlToPath(TestCase):
         self.assertEqual('file:///C:/path/to/f%20oo',
             to_url('C:/path/to/f oo'))
 
+        self.assertEqual('file:///', to_url('/'))
+
         try:
             result = to_url(u'd:/path/to/r\xe4ksm\xf6rg\xe5s')
         except UnicodeError:
             raise TestSkipped("local encoding cannot handle unicode")
 
         self.assertEqual('file:///D:/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s', result)
+        self.assertFalse(isinstance(result, unicode))
 
     def test_win32_unc_path_to_url(self):
         to_url = urlutils._win32_local_path_to_url
@@ -288,7 +407,7 @@ class TestUrlToPath(TestCase):
             raise TestSkipped("local encoding cannot handle unicode")
 
         self.assertEqual('file://HOST/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s', result)
-
+        self.assertFalse(isinstance(result, unicode))
 
     def test_win32_local_path_from_url(self):
         from_url = urlutils._win32_local_path_from_url
@@ -298,6 +417,7 @@ class TestUrlToPath(TestCase):
             from_url('file:///d|/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s'))
         self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d:/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
+        self.assertEqual('/', from_url('file:///'))
 
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
@@ -356,6 +476,53 @@ class TestUrlToPath(TestCase):
         self.assertEqual(('path/..', 'foo'), split('path/../foo'))
         self.assertEqual(('../path', 'foo'), split('../path/foo'))
 
+    def test_split_segment_parameters_raw(self):
+        split_segment_parameters_raw = urlutils.split_segment_parameters_raw
+        self.assertEquals(("/some/path", []),
+            split_segment_parameters_raw("/some/path"))
+        self.assertEquals(("/some/path", ["tip"]),
+            split_segment_parameters_raw("/some/path,tip"))
+        self.assertEquals(("/some,dir/path", ["tip"]),
+            split_segment_parameters_raw("/some,dir/path,tip"))
+        self.assertEquals(("/somedir/path", ["heads%2Ftip"]),
+            split_segment_parameters_raw("/somedir/path,heads%2Ftip"))
+        self.assertEquals(("/somedir/path", ["heads%2Ftip", "bar"]),
+            split_segment_parameters_raw("/somedir/path,heads%2Ftip,bar"))
+        self.assertEquals(("/", ["key1=val1"]),
+            split_segment_parameters_raw(",key1=val1"))
+        self.assertEquals(("foo/", ["key1=val1"]),
+            split_segment_parameters_raw("foo/,key1=val1"))
+        self.assertEquals(("foo/base,la=bla/other/elements", []),
+            split_segment_parameters_raw("foo/base,la=bla/other/elements"))
+        self.assertEquals(("foo/base,la=bla/other/elements", ["a=b"]),
+            split_segment_parameters_raw("foo/base,la=bla/other/elements,a=b"))
+
+    def test_split_segment_parameters(self):
+        split_segment_parameters = urlutils.split_segment_parameters
+        self.assertEquals(("/some/path", {}),
+            split_segment_parameters("/some/path"))
+        self.assertEquals(("/some/path", {"branch": "tip"}),
+            split_segment_parameters("/some/path,branch=tip"))
+        self.assertEquals(("/some,dir/path", {"branch": "tip"}),
+            split_segment_parameters("/some,dir/path,branch=tip"))
+        self.assertEquals(("/somedir/path", {"ref": "heads%2Ftip"}),
+            split_segment_parameters("/somedir/path,ref=heads%2Ftip"))
+        self.assertEquals(("/somedir/path",
+            {"ref": "heads%2Ftip", "key1": "val1"}),
+            split_segment_parameters(
+                "/somedir/path,ref=heads%2Ftip,key1=val1"))
+        self.assertEquals(("/somedir/path", {"ref": "heads%2F=tip"}),
+            split_segment_parameters("/somedir/path,ref=heads%2F=tip"))
+        self.assertEquals(("/", {"key1": "val1"}),
+            split_segment_parameters(",key1=val1"))
+        self.assertEquals(("foo/", {"key1": "val1"}),
+            split_segment_parameters("foo/,key1=val1"))
+        self.assertEquals(("foo/base,key1=val1/other/elements", {}),
+            split_segment_parameters("foo/base,key1=val1/other/elements"))
+        self.assertEquals(("foo/base,key1=val1/other/elements",
+            {"key2": "val2"}), split_segment_parameters(
+                "foo/base,key1=val1/other/elements,key2=val2"))
+
     def test_win32_strip_local_trailing_slash(self):
         strip = urlutils._win32_strip_local_trailing_slash
         self.assertEqual('file://', strip('file://'))
@@ -443,6 +610,10 @@ class TestUrlToPath(TestCase):
     def test_escape(self):
         self.assertEqual('%25', urlutils.escape('%'))
         self.assertEqual('%C3%A5', urlutils.escape(u'\xe5'))
+        self.assertFalse(isinstance(urlutils.escape(u'\xe5'), unicode))
+
+    def test_escape_tildes(self):
+        self.assertEqual('~foo', urlutils.escape('~foo'))
 
     def test_unescape(self):
         self.assertEqual('%', urlutils.unescape('%25'))
@@ -460,7 +631,7 @@ class TestUrlToPath(TestCase):
         def test(expected, base, other):
             result = urlutils.relative_url(base, other)
             self.assertEqual(expected, result)
-            
+
         test('a', 'http://host/', 'http://host/a')
         test('http://entirely/different', 'sftp://host/branch',
                     'http://entirely/different')
@@ -475,7 +646,7 @@ class TestUrlToPath(TestCase):
                     'sftp://host/home/jelmer/branch/2b')
         test('../../branch/feature/%2b', 'http://host/home/jelmer/bar/%2b',
                     'http://host/home/jelmer/branch/feature/%2b')
-        test('../../branch/feature/2b', 'http://host/home/jelmer/bar/2b/', 
+        test('../../branch/feature/2b', 'http://host/home/jelmer/bar/2b/',
                     'http://host/home/jelmer/branch/feature/2b')
         # relative_url should preserve a trailing slash
         test('../../branch/feature/2b/', 'http://host/home/jelmer/bar/2b/',
@@ -493,6 +664,29 @@ class TestUrlToPath(TestCase):
         #test('.', 'http://host/', 'http://host')
         test('http://host', 'http://host/', 'http://host')
 
+        # On Windows file:///C:/path/to and file:///D:/other/path
+        # should not use relative url over the non-existent '/' directory.
+        if sys.platform == 'win32':
+            # on the same drive
+            test('../../other/path',
+                'file:///C:/path/to', 'file:///C:/other/path')
+            #~next two tests is failed, i.e. urlutils.relative_url expects
+            #~to see normalized file URLs?
+            #~test('../../other/path',
+            #~    'file:///C:/path/to', 'file:///c:/other/path')
+            #~test('../../other/path',
+            #~    'file:///C:/path/to', 'file:///C|/other/path')
+
+            # check UNC paths too
+            test('../../other/path',
+                'file://HOST/base/path/to', 'file://HOST/base/other/path')
+            # on different drives
+            test('file:///D:/other/path',
+                'file:///C:/path/to', 'file:///D:/other/path')
+            # TODO: strictly saying in UNC path //HOST/base is full analog
+            # of drive letter for hard disk, and this situation is also
+            # should be exception from rules. [bialix 20071221]
+
 
 class TestCwdToURL(TestCaseInTempDir):
     """Test that local_path_to_url works base on the cwd"""
@@ -506,6 +700,9 @@ class TestCwdToURL(TestCaseInTempDir):
         self.assertEndsWith(url, '/mytest')
 
     def test_non_ascii(self):
+        if win32utils.winver == 'Windows 98':
+            raise TestSkipped('Windows 98 cannot handle unicode filenames')
+
         try:
             os.mkdir(u'dod\xe9')
         except UnicodeError:
@@ -513,9 +710,91 @@ class TestCwdToURL(TestCaseInTempDir):
 
         os.chdir(u'dod\xe9')
 
-        # On Mac OSX this directory is actually: 
+        # On Mac OSX this directory is actually:
         #   u'/dode\u0301' => '/dode\xcc\x81
-        # but we should normalize it back to 
+        # but we should normalize it back to
         #   u'/dod\xe9' => '/dod\xc3\xa9'
         url = urlutils.local_path_to_url('.')
         self.assertEndsWith(url, '/dod%C3%A9')
+
+
+class TestDeriveToLocation(TestCase):
+    """Test that the mapping of FROM_LOCATION to TO_LOCATION works."""
+
+    def test_to_locations_derived_from_paths(self):
+        derive = urlutils.derive_to_location
+        self.assertEqual("bar", derive("bar"))
+        self.assertEqual("bar", derive("../bar"))
+        self.assertEqual("bar", derive("/foo/bar"))
+        self.assertEqual("bar", derive("c:/foo/bar"))
+        self.assertEqual("bar", derive("c:bar"))
+
+    def test_to_locations_derived_from_urls(self):
+        derive = urlutils.derive_to_location
+        self.assertEqual("bar", derive("http://foo/bar"))
+        self.assertEqual("bar", derive("bzr+ssh://foo/bar"))
+        self.assertEqual("foo-bar", derive("lp:foo-bar"))
+
+
+class TestRebaseURL(TestCase):
+    """Test the behavior of rebase_url."""
+
+    def test_non_relative(self):
+        result = urlutils.rebase_url('file://foo', 'file://foo',
+                                     'file://foo/bar')
+        self.assertEqual('file://foo', result)
+        result = urlutils.rebase_url('/foo', 'file://foo',
+                                     'file://foo/bar')
+        self.assertEqual('/foo', result)
+
+    def test_different_ports(self):
+        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+                              'foo', 'http://bar:80', 'http://bar:81')
+        self.assertEqual(str(e), "URLs differ by more than path:"
+                         " 'http://bar:80' and 'http://bar:81'")
+
+    def test_different_hosts(self):
+        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+                              'foo', 'http://bar', 'http://baz')
+        self.assertEqual(str(e), "URLs differ by more than path: 'http://bar'"
+                         " and 'http://baz'")
+
+    def test_different_protocol(self):
+        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+                              'foo', 'http://bar', 'ftp://bar')
+        self.assertEqual(str(e), "URLs differ by more than path: 'http://bar'"
+                         " and 'ftp://bar'")
+
+    def test_rebase_success(self):
+        self.assertEqual('../bar', urlutils.rebase_url('bar', 'http://baz/',
+                         'http://baz/qux'))
+        self.assertEqual('qux/bar', urlutils.rebase_url('bar',
+                         'http://baz/qux', 'http://baz/'))
+        self.assertEqual('.', urlutils.rebase_url('foo',
+                         'http://bar/', 'http://bar/foo/'))
+        self.assertEqual('qux/bar', urlutils.rebase_url('../bar',
+                         'http://baz/qux/foo', 'http://baz/'))
+
+    def test_determine_relative_path(self):
+        self.assertEqual('../../baz/bar',
+                         urlutils.determine_relative_path(
+                         '/qux/quxx', '/baz/bar'))
+        self.assertEqual('..',
+                         urlutils.determine_relative_path(
+                         '/bar/baz', '/bar'))
+        self.assertEqual('baz',
+                         urlutils.determine_relative_path(
+                         '/bar', '/bar/baz'))
+        self.assertEqual('.', urlutils.determine_relative_path(
+                         '/bar', '/bar'))
+
+
+class TestParseURL(TestCase):
+
+    def test_parse_url(self):
+        self.assertEqual(urlutils.parse_url('http://example.com:80/one'),
+            ('http', None, None, 'example.com', 80, '/one'))
+        self.assertEqual(urlutils.parse_url('http://[1:2:3::40]/one'),
+                ('http', None, None, '1:2:3::40', None, '/one'))
+        self.assertEqual(urlutils.parse_url('http://[1:2:3::40]:80/one'),
+                ('http', None, None, '1:2:3::40', 80, '/one'))
