@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,14 +39,14 @@ from bzrlib.trace import mutter
 from bzrlib.transport import LateReadError
 """)
 
-from bzrlib.transport import Transport, Server
+from bzrlib import transport
 
 
-_append_flags = os.O_CREAT | os.O_APPEND | os.O_WRONLY | osutils.O_BINARY
-_put_non_atomic_flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | osutils.O_BINARY
+_append_flags = os.O_CREAT | os.O_APPEND | os.O_WRONLY | osutils.O_BINARY | osutils.O_NOINHERIT
+_put_non_atomic_flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | osutils.O_BINARY | osutils.O_NOINHERIT
 
 
-class LocalTransport(Transport):
+class LocalTransport(transport.Transport):
     """This is the transport agent for local filesystem access."""
 
     def __init__(self, base):
@@ -160,7 +160,7 @@ class LocalTransport(Transport):
             transport._file_streams[canonical_url].flush()
         try:
             path = self._abspath(relpath)
-            return open(path, 'rb')
+            return osutils.open_file(path, 'rb')
         except (IOError, OSError),e:
             if e.errno == errno.EISDIR:
                 return LateReadError(relpath)
@@ -329,7 +329,7 @@ class LocalTransport(Transport):
         # initialise the file
         self.put_bytes_non_atomic(relpath, "", mode=mode)
         abspath = self._abspath(relpath)
-        handle = open(abspath, 'wb')
+        handle = osutils.open_file(abspath, 'wb')
         if mode is not None:
             self._check_mode_and_size(abspath, handle.fileno(), mode)
         transport._file_streams[self.abspath(relpath)] = handle
@@ -399,10 +399,12 @@ class LocalTransport(Transport):
 
     def rename(self, rel_from, rel_to):
         path_from = self._abspath(rel_from)
+        path_to = self._abspath(rel_to)
         try:
             # *don't* call bzrlib.osutils.rename, because we want to
-            # detect errors on rename
-            os.rename(path_from, self._abspath(rel_to))
+            # detect conflicting names on rename, and osutils.rename tries to
+            # mask cross-platform differences there
+            os.rename(path_from, path_to)
         except (IOError, OSError),e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)
@@ -481,7 +483,7 @@ class LocalTransport(Transport):
         path = relpath
         try:
             path = self._abspath(relpath)
-            return os.stat(path)
+            return os.lstat(path)
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
@@ -515,6 +517,33 @@ class LocalTransport(Transport):
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
+    if osutils.host_os_dereferences_symlinks():
+        def readlink(self, relpath):
+            """See Transport.readlink."""
+            return osutils.readlink(self._abspath(relpath))
+
+    if osutils.hardlinks_good():
+        def hardlink(self, source, link_name):
+            """See Transport.link."""
+            try:
+                os.link(self._abspath(source), self._abspath(link_name))
+            except (IOError, OSError), e:
+                self._translate_error(e, source)
+
+    if osutils.has_symlinks():
+        def symlink(self, source, link_name):
+            """See Transport.symlink."""
+            abs_link_dirpath = urlutils.dirname(self.abspath(link_name))
+            source_rel = urlutils.file_relpath(
+                urlutils.strip_trailing_slash(abs_link_dirpath),
+                urlutils.strip_trailing_slash(self.abspath(source))
+            )
+
+            try:
+                os.symlink(source_rel, self._abspath(link_name))
+            except (IOError, OSError), e:
+                self._translate_error(e, source_rel)
+
     def _can_roundtrip_unix_modebits(self):
         if sys.platform == 'win32':
             # anyone else?
@@ -554,26 +583,7 @@ class EmulatedWin32LocalTransport(LocalTransport):
             return EmulatedWin32LocalTransport(abspath)
 
 
-class LocalURLServer(Server):
-    """A pretend server for local transports, using file:// urls.
-
-    Of course no actual server is required to access the local filesystem, so
-    this just exists to tell the test code how to get to it.
-    """
-
-    def setUp(self):
-        """Setup the server to service requests.
-
-        :param decorated_transport: ignored by this implementation.
-        """
-
-    def get_url(self):
-        """See Transport.Server.get_url."""
-        return urlutils.local_path_to_url('')
-
-
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    return [
-            (LocalTransport, LocalURLServer),
-            ]
+    from bzrlib.tests import test_server
+    return [(LocalTransport, test_server.LocalURLServer),]
